function varargout = plot_basic(varargin)

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @plot_basic_OpeningFcn, ...
                   'gui_OutputFcn',  @plot_basic_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

% *************************************************************************
%  GUI Initialization - runs once!
% *************************************************************************
function plot_basic_OpeningFcn(hObject, eventdata, h, varargin)
     
    % Clear matlab terminal
    clc
        
    % Get arugments from live script, implied order:
    % diameter,k,length,temp left, temp right,resistance1,resistance2
    h.invars=cell2mat(varargin);
    
    % Check for number of arugments
    if(length(h.invars)~=7)
       disp('Need exactly 7 arguments')
       disp('Using hard-coded vars!')
      h.invars(1:7)=[0.01,250.0,0.1,1000.0,500.0,2.0,4.0];
    end
   
    plot_data_init(hObject,h);
    
end

end

% *************************************************************************
%  Data Initalization - Called at start and reset
% *************************************************************************
function plot_data_init(hObject,h)
    
    % Defined Global Constants
    h.nxi=20;
    nxi=h.nxi;
    h.neta=20;
    neta=h.neta;
    
    % Set multipliers, values greater than 1 only
    h.mult_limit(1,1:4)=[2,2,2,2]; % set diameter limits
    h.mult_limit(2,1:4)=[2,2,2,2]; % set temp/resistance limits
    h.mult_limit(3,1:4)=[2,2,2,0]; % set conductivity limits
    h.mult_limit(4,1:4)=[2,2,2,0]; % set length limits   
    
    % initialize node parameters (1:4)
    h.diam(1:4)=h.invars(1); %diameter
    h.area(1:4)=pi.*((0.5.*h.diam(1:4)).^2); % area
    h.tend(1:2)=h.invars(4:5); % area
    h.tohm(1:2)=h.invars(6:7); % thermal resistance
    
    % intiailize beam parameters (1:3)
    h.karray=[0.02, 0.05, 0.1, 1.5, 2.22, 50.2, 237, 310, 401, 1000, 10000];
    h.k(1:3)=h.invars(2);  % conductivity
    h.len(1:3)=h.invars(3);  % conductivity
    
    % Initialize mesh and surf
    h.ex=zeros(neta,nxi,3);
    h.ey=zeros(neta,nxi,3);
    h.ez=zeros(neta,nxi,3);
    
    % Initialize output vars
    h.etemp=zeros(neta,nxi,3);
    h.eflux=zeros(neta,nxi,3);
    h.erate=zeros(neta,nxi,3);
    h.earea=zeros(1,nxi,3);
    
    %zero intermediate tems
    h.tx(1:4)=0.0;
    
    % update totale area
    h.earea(1,:,:)=pi*((0.5*h.diam(1))^2);
    
    % Define handle array (row on gui:column on gui)
    % note sparse in element rows!
    h.sliders=[h.d1_slider,h.d2_slider,h.d3_slider,h.d4_slider;...
               h.t1_slider,h.t2_slider,h.t3_slider,h.t4_slider;...
               h.e1k_slider,h.e2k_slider,h.e3k_slider,h.e1k_slider;...
               h.e1x_slider,h.e2x_slider,h.e3x_slider,h.e1x_slider];
           
    h.lives=[h.d1_live,h.d2_live,h.d3_live,h.d4_live;...
             h.t1_live,h.t2_live,h.t3_live,h.t4_live;...
             h.e1k_live,h.e2k_live,h.e3k_live,h.e1k_live;...
             h.e1x_live,h.e2x_live,h.e3x_live,h.e1x_live];   
    
    h.lists=[h.e1k_list,h.e2k_list,h.e3k_list];
         
    % get initialization matrix (local scope)
    init(1,1:4)=h.diam(1:4);
    init(2,1:4)=[h.tend(1),h.tohm(1:2),h.tend(2)];
    init(3,1:4)=[h.k(1:3),0];
    init(4,1:4)=[h.len(1:3),0];
     
    % Initialize gui sliders
    for j=1:4 % rows
        for i=1:4 % columns
            if (j<=2 || i<4)
                set(h.sliders(j,i),'Max',init(j,i)*h.mult_limit(j,i));
                set(h.sliders(j,i),'Min',init(j,i)/h.mult_limit(j,i));
                set(h.sliders(j,i),'Value',init(j,i));
                set(h.lives(j,i),'String',num2str(init(j,i)));
            end
        end
    end
    
    % set popup initial states
    set(h.lists(1),'Value',12);
    set(h.lists(2),'Value',12);
    set(h.lists(3),'Value',12);
    
    % Set plot limints
    h.gmax(1)=get(h.sliders(4,1),'Max'); % x limit
    h.gmax(2)=get(h.sliders(1,1),'Max'); % y limit
    if (h.tend(1)>h.tend(2)) % get from t_left
        h.gmax(3)=get(h.sliders(2,1),'Max'); % z colormap        
    else
        h.gmax(3)=get(h.sliders(2,4),'Max'); % z colormap
    end

    % Intiailize plots
    axes(h.color_fig);
    hold on
    h.plot1=surf(h.ex(:,:,1)',h.ey(:,:,1)',h.ez(:,:,1)','edgecolor','none');
    h.plot2=surf(h.ex(:,:,2)',h.ey(:,:,2)',h.ez(:,:,2)','edgecolor','none');
    h.plot3=surf(h.ex(:,:,3)',h.ey(:,:,3)',h.ez(:,:,3)','edgecolor','none');
    hold off
    
    % set initial colormap
    h.ztype=1;
    set(h.select_colormap,'Value',1);
    
    % Intial generation calls 
    % Calculate new x/y mesh
    h=update_mesh(h);
    
    % Get new data in ez1:3
    % Should implicity update live fields
    h=update_data(h);
    
    % Generate new plots
    hold all
    h=update_plot(h,1);
        
    % Choose default command line output for plot_basic
    h.output = hObject;
    
    % Update h structure
    guidata(hObject, h);
    
end


% *************************************************************************
%  MESH UPDATE - update mesh
% *************************************************************************
function h=update_mesh(h)   
    
    h=xy_gen(h,1); % left
    h=xy_gen(h,2); % mid
    h=xy_gen(h,3); % right

end

function h=xy_gen(h,idx)
 
    % Get base x
    if (idx>1)
        x_base=h.ex(1,h.nxi,idx-1);
    else
        x_base=0;
    end
    
    % Get constant delta x
    dx=h.len(idx)/(h.nxi-1);
    
    % Loop x, define xall, loop y
    for i=1:h.nxi
        
        % define all x for given x
        h.ex(:,i,idx)=x_base+(i-1)*dx;
        
        % get y displacement, unique at each i
        y_base=(h.diam(idx+1)-h.diam(idx))/h.len(idx); % slope
        y_base=y_base*(h.ex(1,i,idx)-x_base); %displacement to point 
        y_base=y_base+h.diam(idx); %add to left y
        dy=y_base/(h.neta-1); % divide by num-1;
        
        % update totale area
        h.earea(1,i,idx)=pi*((0.5*y_base)^2);
        
        % update y base for graphical offset
        y_base=-y_base*0.5;
        
        % loop y and define
        for j=1:h.neta
            h.ey(j,i,idx)=y_base+(j-1)*dy;
        end
        
    end
    
end

% *************************************************************************
%  DATA UPDATE - Generate new data piecmeal
% *************************************************************************
function h=update_data(h)
     
    % Solve for temperature conditions - h.tx(1:4)
    h=get_temps_bc(h);
    
    % Get all three temeprature profiles - in h.ez(eta,xi,idx)
    h=get_temps_all(h,1);
    h=get_temps_all(h,2);
    h=get_temps_all(h,3);
    
end

function h=get_temps_all(h,idx)
    
    % Select Bound Data
    if (idx==1) 
        t1=h.tend(1);
        t2=h.tx(1);
        ar=h.area(2)/h.area(1);
        xroot=0.0;
    elseif(idx==2)
        t1=h.tx(2);
        t2=h.tx(3);
        ar=h.area(3)/h.area(2);  
        xroot=h.ex(1,end,1);
    else
        t1=h.tx(4);
        t2=h.tend(2);
        ar=h.area(4)/h.area(3);      
        xroot=h.ex(1,end,2);
    end
    
    % Get temperature profile
    if (ar==1.0) % equal areas
        for i=1:h.nxi
            f1=t2-t1;
            h.etemp(:,i,idx)=t1+((h.ex(:,i,idx)-xroot)/h.len(idx))*f1;
            h.eflux(:,i,idx)=-h.k(idx)*(t2-t1)/h.len(idx);           
            
% % % % % % %     SIPPEL CHANGED FOLLOWING LINE.             
%             h.erate(:,i,idx)=h.eflux(:,i,idx)*h.earea(1,i,idx);
            h.erate(:,i,idx)=h.eflux(1,1,1)*h.earea(1,1,1);
            
        end
    else %normal
        f1=(t2-t1)/log(ar);
        f2=(ar-1)/h.len(idx);
        f3=ar-1.0;
        for i=1:h.nxi
            h.etemp(:,i,idx)=t1+f1*log(1.0+(h.ex(:,i,idx)-xroot)*f2);
            h.eflux(:,i,idx)=-h.k(idx)*f1*(f3/(h.len(idx)+(h.ex(:,i,idx)-xroot)*f3));

% % % % % % %     SIPPEL CHANGED FOLLOWING LINE.                         
%             h.erate(:,i,idx)=h.eflux(:,i,idx)*h.earea(1,i,idx);
            h.erate(:,i,idx)=h.eflux(1,1,1)*h.earea(1,1,1);
            
        end
    end

end

function h=get_temps_bc(h)

    % solution referenece
        %  (A*B*T1 + A*C*T1 + B*C*T4 - A*B*C*Rt2*T1 - A*B*C*Rt3*T1)
        %  (A*B*T1 + A*C*T1 + B*C*T4 - A*B*C*Rt3*T1 - A*B*C*Rt2*T4)
        %  (A*B*T1 + A*C*T4 + B*C*T4 - A*B*C*Rt3*T1 - A*B*C*Rt2*T4)
        %  (A*B*T1 + A*C*T4 + B*C*T4 - A*B*C*Rt2*T4 - A*B*C*Rt3*T4)
        % div num/ (A*B + A*C + B*C - A*B*C*Rt2 - A*B*C*Rt3)
%         
%  (A*B*T1 + A*C*T1 + B*C*T4 + A*B*C*T1*rt2 + A*B*C*T1*rt3)/(A*B + A*C + B*C + A*B*C*rt2 + A*B*C*rt3)
%  (A*B*T1 + A*C*T1 + B*C*T4 + A*B*C*T1*rt3 + A*B*C*T4*rt2)/(A*B + A*C + B*C + A*B*C*rt2 + A*B*C*rt3)
%  (A*B*T1 + A*C*T4 + B*C*T4 + A*B*C*T1*rt3 + A*B*C*T4*rt2)/(A*B + A*C + B*C + A*B*C*rt2 + A*B*C*rt3)
%  (A*B*T1 + A*C*T4 + B*C*T4 + A*B*C*T4*rt2 + A*B*C*T4*rt3)/(A*B + A*C + B*C + A*B*C*rt2 + A*B*C*rt3)
 
    % Define coefficient left - avoid div by 0
    if (h.area(2)==h.area(1)) % avoid div0
        A=h.k(1)*h.area(2)/h.len(1);
    else
        A=h.k(1)*(h.area(2)-h.area(1))/(h.len(1)*log(h.area(2)/h.area(1)));   
    end
    
    % Define coefficient Middle - avoid div by 0
    if (h.area(3)==h.area(2)) % avoid div0
        B=h.k(2)*h.area(3)/h.len(2);
    else
        B=h.k(2)*(h.area(3)-h.area(2))/(h.len(2)*log(h.area(3)/h.area(2)));
    end
    
    % Define coefficient Middle - avoid div by 0
    if (h.area(4)==h.area(3)) % avoid div0
        C=h.k(3)*h.area(4)/h.len(3);
    else
        C=h.k(3)*(h.area(4)-h.area(3))/(h.len(3)*log(h.area(4)/h.area(3)));
    end
    
    % Common terms
   n1=A*B*h.tend(1);
   n2ab=A*C*h.tend(1);
   n2cd=A*C*h.tend(2);
   n3=B*C*h.tend(2);
   abc=A*B*C;
   
   % Get Numerator
   h.tx(1)=n1+n2ab+n3+abc*h.tohm(1)*h.tend(1)+abc*h.tohm(2)*h.tend(1); %T2a
   h.tx(2)=n1+n2ab+n3+abc*h.tohm(2)*h.tend(1)+abc*h.tohm(1)*h.tend(2); %t2b
   h.tx(3)=n1+n2cd+n3+abc*h.tohm(2)*h.tend(1)+abc*h.tohm(1)*h.tend(2); %t3a
   h.tx(4)=n1+n2cd+n3+abc*h.tohm(1)*h.tend(2)+abc*h.tohm(2)*h.tend(2); %t3b
   
   % divide by denominator
   h.tx(:)=h.tx(:)/(A*B + A*C + B*C + abc*h.tohm(1) + abc*h.tohm(2));

end
% *************************************************************************
%  PLOT UPDATE - Generate new plot
% *************************************************************************
function h=update_plot(h,idx)

    % Select color axes
    axes(h.color_fig);    
    
    % select z type
    if (h.ztype==1) % temperature
        h.ez(:,:,:)=h.etemp(:,:,:);
        h.gmax(3)=max([get(h.sliders(2,1),'Max'),get(h.sliders(2,4),'Max')]); % templimits
        caxis(h.color_fig,[0 h.gmax(3)]);
    elseif (h.ztype==2) % heat flux
        h.ez(:,:,:)=h.eflux(:,:,:);
        caxis(h.color_fig,[min(min(min(h.ez(:,:,:)))),max(max(max(h.ez(:,:,:))))]);
    elseif (h.ztype==3) % heat flow
        h.ez(:,:,:)=h.erate(:,:,:);
        caxis(h.color_fig,[min(min(min(h.ez(:,:,:)))),max(max(max(h.ez(:,:,:))))]);
    end
    
    % Update Data
    hold on
    set(h.plot1,'XData',h.ex(:,:,1));
    set(h.plot1,'YData',h.ey(:,:,1));
    set(h.plot1,'ZData',h.ez(:,:,1));     

    set(h.plot2,'XData',h.ex(:,:,2));
    set(h.plot2,'YData',h.ey(:,:,2));
    set(h.plot2,'ZData',h.ez(:,:,2));        

    set(h.plot3,'XData',h.ex(:,:,3));
    set(h.plot3,'YData',h.ey(:,:,3));
    set(h.plot3,'ZData',h.ez(:,:,3));        
    hold off

    % plot bounds
    h.gmax(1)=sum([get(h.sliders(4,1),'Max'),get(h.sliders(4,2),'Max'),get(h.sliders(4,3),'Max')]); % templimits
    h.gmax(2)=max([get(h.sliders(1,1),'Max'),get(h.sliders(1,2),'Max'),get(h.sliders(1,3),'Max'),get(h.sliders(1,4),'Max')]); % templimits
    
    % Set limits
    xlim(h.color_fig,[0 h.gmax(1)]);
    ylim(h.color_fig,[-0.5*h.gmax(2) 0.5*h.gmax(2)]);  
    colormap(h.color_fig,'hot');
    grid on
%     set(h.plot1,'CLim',[0 h.gmax(3)]);
%     set(h.plot2,'XLim',[0 h.gmax(1)]);
%     set(h.plot2,'YLim',[-0.5*h.gmax(2) 0.5*h.gmax(2)]);  
%     set(h.plot2,'CLim',[0 h.gmax(3)]);    set(h.plot1,'XLim',[0 h.gmax(1)]);
%     set(h.plot3,'XLim',[0 h.gmax(1)]);
%     set(h.plot3,'YLim',[-0.5*h.gmax(2) 0.5*h.gmax(2)]);  
%     set(h.plot3,'CLim',[0 h.gmax(3)]);    
%     
%     caxis([0 h.gmax(3)]);
    view(0,90);
    
    % Select data axes
    axes(h.data_fig);   
    grid on
    
    % select left side
    yyaxis left
    plot( [h.ex(1,:,1),h.ex(1,:,2),h.ex(1,:,3)],[h.etemp(1,:,1), h.etemp(1,:,2), h.etemp(1,:,3)] );
    ylabel('Temperature (K)');
    
    % select right side
    yyaxis right
    plot( [h.ex(1,:,1),h.ex(1,:,2),h.ex(1,:,3)],[h.earea(1,:,1), h.earea(1,:,2), h.earea(1,:,3)] );    
    ylabel('Area (m^2)');
    
    % Select data axes
    axes(h.data_fig2);   
    grid on
    xlabel('Length (m)');
    
    % select left side
    yyaxis left
    plot( [h.ex(1,:,1),h.ex(1,:,2),h.ex(1,:,3)],[h.eflux(1,:,1), h.eflux(1,:,2), h.eflux(1,:,3)] );
    ylabel('Heat Flux (W/m^2)');
    
    % select right side
    yyaxis right
    
    
% % % % % % %     SIPPEL ADDED FOLLOWING LINES. 
    h.erate(1,:,1) = h.eflux(1,1,1) .* h.earea(1,1,1);
    h.erate(1,:,2) = h.eflux(1,1,1) .* h.earea(1,1,1);
    h.erate(1,:,3) = h.eflux(1,1,1) .* h.earea(1,1,1);
    

    plot( [h.ex(1,:,1),h.ex(1,:,2),h.ex(1,:,3)],[h.erate(1,:,1), h.erate(1,:,2), h.erate(1,:,3)] );    
    ylabel('Heat Rate (W)');
    
end

% *************************************************************************
%  diameter Definition Callbacks
% *************************************************************************
function h=update_diam(hObject, eventdata,h,idx,dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.diam(idx)=get(h.sliders(1,idx),'Value');
    else % live text
        h.diam(idx)=str2num(get(h.lives(1,idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype,1);
        
    % update area
    h.area(idx)=pi.*((0.5.*h.diam(idx)).^2); % area
    
    % call new mesh on local segment
    h=update_mesh(h);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h,idx);
        
    % Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Contact Resistance 
% *************************************************************************
function h=update_resistance(hObject, eventdata, h,idx,dtype)

    % remap idx
    idx2=1;
    if (idx>2) 
        idx2=2;
    end
    
    % update diamater & area
    if (dtype==0) % slider
        h.tohm(idx2)=get(h.sliders(2,idx),'Value');
    else % live text
        h.tohm(idx2)=str2num(get(h.lives(2,idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype,2);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h,idx);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Temperature Boundaries 
% *************************************************************************
function h=update_temp(hObject, eventdata, h, idx, dtype)

    % remap idx
    idx_local=idx;
    if (idx>1) 
        idx_local=2;
    end
    
    % update diamater & area
    if (dtype==0) % slider
        h.tend(idx_local)=get(h.sliders(2,idx),'Value');
    else % live text
        h.tend(idx_local)=str2num(get(h.lives(2,idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype,2);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h,idx);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Conductivity Values
% *************************************************************************
function h=update_conductivity(hObject, eventdata, h, idx, dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.k(idx)=get(h.sliders(3,idx),'Value');
    elseif (dtype==2) %list box
        h.k(idx)=klookup(get(h.lists(idx),'Value'),h);
    else % live text
        h.k(idx)=str2num(get(h.lives(3,idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype,3);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h,idx);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Length Values
% *************************************************************************
function h=update_length(hObject, eventdata, h, idx, dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.len(idx)=get(h.sliders(4,idx),'Value');
    else % live text
        h.len(idx)=str2num(get(h.lives(4,idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype,4);
        
    % call new mesh on local segment
    h=update_mesh(h);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h,idx);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
% *************************************************************************
%  Utility Functions
% *************************************************************************
% *************************************************************************

% *************************************************************************
% Pick color map
% *************************************************************************
function h=select_colormap_Callback(hObject, eventdata, h)

    %update plot type
    h.ztype=get(h.select_colormap,'Value');
    
    % call new mesh on local segment
    h=update_mesh(h);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h,1);
   
    
end

% *************************************************************************
% Generate conductiivty data
% *************************************************************************
function value=klookup(idx,h)

    % if custom, display original value
    if (idx>=length(h.karray))
        value=h.invars(2);
    else % get value from array
        value=h.karray(idx);
    end
    
end

% *************************************************************************
%  Export to CSV
% *************************************************************************
function export_to_csv_Callback(hObject, eventdata, h)
end

% *************************************************************************
%  Export to workspace
% *************************************************************************
function export_to_workspace_Callback(hObject, eventdata, h)
end

% *************************************************************************
%  Take Screenshot
% *************************************************************************
function screenshot_button_Callback(hObject, eventdata, h)
end

% *************************************************************************
%  Reset Data
% *************************************************************************
function reset_data_Callback(hObject, eventdata, h)
    delete(h.plot1);
    delete(h.plot2)
    delete(h.plot3)
	plot_data_init(hObject,h);
end

% *************************************************************************
%  Make sure live text - slider - and popups all work in sync
% *************************************************************************
function h=update_entry(hObject, eventdata, h, idx, dtype, row)
    
    % if slider was changed
    if (dtype==0) % update live text, maybe popup
       
        % get value
        val=get(h.sliders(row,idx),'Value');
        
        % update live text
        set(h.lives(row,idx),'String',num2str2(val));
        
        % if thermal conducivity, set to custom
        if (row==3)
            set(h.lists(idx),'Value',12);
        end
        
    elseif (dtype==1) % if live was changed, update slider, maybe popup
        
        % get value
        val=str2num(get(h.lives(row,idx),'String'));
        
        % reset slider range if out of bounds
        h=slider_update(val,row,idx,h);
        
        % if thermal conducitivty, set to custom
        if (row==3)
            set(h.lists(idx),'Value',12);
        end
        
    else % popup changed
        
        % get value
        val=klookup(get(h.lists(idx),'Value'),h);
        
        % reset slider range if out of bounds
        h=slider_update(val,row,idx,h);
        
        % fill live test field
        set(h.lives(row,idx),'String',num2str2(val));
        
    end
        
end

% *************************************************************************
%  Used in update_entry, common routine for updating sliders
% *************************************************************************
function h=slider_update(val,row,idx,h)
       
    % Get current rangers
    max_slider=get(h.sliders(row,idx),'Max');
    min_slider=get(h.sliders(row,idx),'Min');
    
    % If new value is out of bounds
    if (val>max_slider) % new value too big
        set(h.sliders(row,idx),'Max',val*h.mult_limit(row,idx)); 
        set(h.sliders(row,idx),'Value',val);
        set(h.sliders(row,idx),'Min',val/h.mult_limit(row,idx));
        
    elseif (val<min_slider) % New value too small
        set(h.sliders(row,idx),'Min',val/h.mult_limit(row,idx));
        set(h.sliders(row,idx),'Value',val);
        set(h.sliders(row,idx),'Max',val*h.mult_limit(row,idx));
        
    else % New value jsut right
        set(h.sliders(row,idx),'Value',val);
    end 

end

% *************************************************************************
%  Live Text - proper precision
% *************************************************************************
function str=num2str2(val)

    % Determine precision type
    if (val>10.0) % no decimals
        str=num2str(val,'%.0f');
    elseif (val>1.0) % 1 deciaml
        str=num2str(val,'%.1f');
    else % 8 preciision
        str=num2str(val,'%.3g');
    end
        
end

% maight be able to delete, returns to command line
function varargout = plot_basic_OutputFcn(hObject, eventdata, h) 
varargout{1} = 0;
end
