function varargout = plot_basic(varargin)

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @plot_basic_OpeningFcn, ...
                   'gui_OutputFcn',  @plot_basic_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

% *************************************************************************
%  GUI Initialization - runs once!
% *************************************************************************
function plot_basic_OpeningFcn(hObject, eventdata, h, varargin)
     
    % Clear matlab terminal
%     clc
        
    % Get arugments from live script, implied order:
    % diameter,k,length,temp left, temp right,resistance1,resistance2
    if length(varargin)==9
        h.invars=cell2mat(varargin);
    else
       h.invars(1:9)=[0.1,0.025,0.1,0.2,2.0,1000,300,1,1];
    end
   
    plot_data_init(hObject,h);
    
end

end

% *************************************************************************
%  Data Initalization - Called at start and reset
% *************************************************************************
function plot_data_init(hObject,h)
    
    % Defined Global Constants
    h.nxi=40;
    nxi=h.nxi;
    h.neta=20;
    neta=h.neta;
    
    % Plot Indexing
    % 1 = Area
    % 2 = Aspect Ratio
    % 3 = Length
    % 4 = Material Conductivity
    % 5 = Convective Heat Transfer Coefficient
    % 6 = Air Temp (Tinf)
    % 7 = Base Temp (Tb)
    
    % Set multipliers, values greater than 1 only
    h.mult_limit(1:9)=[2,2,2,2,2,2,2,2,2]; % set diameter limits
    
    % Initialize conductivity
    h.karray=[0.02, 0.05, 0.1, 1.5, 2.22, 50.2, 237, 310, 401, 1000, 10000];
    
    % initialize node parameters (1:4)
    h.conv=h.invars(1); % H-coeff
    h.area=h.invars(2); % Area
    h.arat=h.invars(3); % Aspect Ratio
    h.leng=h.invars(4); % Length
    h.cond=h.invars(5); % Material Cond. 
    h.tbas=h.invars(6); % Base Temp
    h.tair=h.invars(7); % Air Temp
    h.atyp=h.invars(8); % Cross Section shape (1=tri,2=rect,3=ellipse)
    h.bctp=h.invars(9); % Tip BC (1=conv,2=adiabatic,3=fixedT,4=infinite)
    h.tend=300.0;
    
    % Initialize mesh and surf
    h.ex=zeros(neta,nxi);
    h.ey=zeros(neta,nxi);
    h.ez=zeros(neta,nxi);
    
    % Initialize output vars
    h.etemp=zeros(nxi,1);
    h.eflux=zeros(nxi,1);
    h.qf=0.0;
    h.ef=0.0;
    h.nf=0.0;
    
    % Define handle array (row on gui:column on gui)
    % note sparse in element rows!
    h.sliders=[h.area_slider,h.arat_slider,h.leng_slider,h.cond_slider,...
               h.conv_slider,h.tair_slider,h.tbas_slider,h.tend_slider,0];
           
    h.lives=[h.area_live,h.arat_live,h.leng_live,h.cond_live,...
               h.conv_live,h.tair_live,h.tbas_live,h.tend_live,0]; 
    
    h.lists=[0,0,0,h.cond_list,0,0,0,0,0];
    
    set(h.lists(4),'Value',12);
    
    % get initialization matrix (local scope)
    init(1:8)=[h.invars(2:5),h.invars(1),h.invars(6:7),h.tend];
    for j=1:8
        set(h.sliders(j),'Max',init(j)*h.mult_limit(j));
        set(h.sliders(j),'Min',init(j)/h.mult_limit(j));
        set(h.sliders(j),'Value',init(j));
        set(h.lives(j),'String',num2str(init(j)));
    end
    
    % Set selection values
%     h.select=[h.ar_set,h.bc_set];    
    
    % Set selections
%     for j=1:2
%        set(h.select(j),) 
%     end
    
    % Set plot limints
%     h.gmax(1)=get(h.sliders(3),'Max'); % x limit
%     h.gmax(2)=get(h.diam,'Max'); % y limit
%     if (h.tend(1)>h.tend(2)) % get from t_left
%         h.gmax(3)=get(h.sliders(2,1),'Max'); % z colormap        
%     else
%         h.gmax(3)=get(h.sliders(2,4),'Max'); % z colormap
%     end
    
%     % Intiailize plots
    axes(h.color_fig);
    hold on
    grid on
    title('Heat Map')
    h.plot1=surf(h.ex(:,:),h.ey(:,:),h.ez(:,:),'edgecolor','none');
    hold off
    
    axes(h.data_fig2);
    grid on
    title('Cross-Section')
    xlabel('meter')
    ylabel('meter')
    axis square
     
    % GEt cross section
    h=get_cross_section(h);
    
    % Intial generation calls 
    % Calculate new x/y mesh
    h=update_mesh(h);
    
    % Get new data in ez1:3
    % Should implicity update live fields
    h=update_data(h);
    
    % Generate new plots
    h=update_plot(h);
        
    % Choose default command line output for plot_basic
    h.output = hObject;
    
    % Update h structure
    guidata(hObject, h);
    
end


% *************************************************************************
%  MESH UPDATE - update mesh
% *************************************************************************
function h=update_mesh(h)
    
    % Define x
    % *********************************
    % Get constant delta x
    dx=h.leng/(h.nxi-1);
    
    % Loop x, define xall, loop y
    for i=1:h.nxi
        h.ex(:,i)=(i-1)*dx;
    end
    
    % get y displacement, unique at each i
    dy=h.diam/(h.neta-1); % divide by num-1;
    
    % update y base for graphical offset
    y_base=-h.diam*0.5;
    
    % loop y and define
    for j=1:h.neta
        h.ey(j,:)=y_base+(j-1)*dy;
    end
    
end

% *************************************************************************
%  DATA UPDATE - Generate new data piecmeal
% *************************************************************************
function h=update_data(h)
    
    % Get coefficients
    m=sqrt((h.conv*h.perim)/(h.cond*h.area));
    M=sqrt(h.conv*h.perim*h.cond*h.area)*(h.tbas-h.tair);
    x=h.ex(1,:)';
    l=h.leng;
    
    % Get temp, heat flux
    if (h.bctp==1) % convection BC
        a=(h.conv/(h.cond*m));
        h.etemp(1:h.nxi)=cosh(m*(l-x));
        h.etemp(1:h.nxi)=h.etemp(1:h.nxi)+a*sinh(m*(l-x));
        h.etemp(1:h.nxi)=h.etemp(1:h.nxi)/(cosh(m*l)+(a*sinh(m*l)));      
        h.qf=M*(sinh(m*l)+a*cosh(m*l));
        h.qf=h.qf/(cosh(m*l)+a*sinh(m*l));
        
        % Get flux
        h.eflux=h.cond.*m.*(h.tbas-h.tair);
        h.eflux=h.eflux.*(sinh(m*(l-x))+a*cosh(m*(l-x)));
        h.eflux=h.eflux./(cosh(m.*l)+a.*sinh(m.*l)); 
        
    elseif (h.bctp==2) % adiabatic
        h.etemp(1:h.nxi)=cosh(m*(l-x));
        h.etemp(1:h.nxi)=h.etemp(1:h.nxi)/cosh(m*l);
        h.qf=M*tanh(m*l);
        
        % flux
        h.eflux=h.cond.*m.*(h.tbas-h.tair)*(sinh(m*(l-x)));
        h.eflux=h.eflux./cosh(m.*l);
        
    elseif (h.bctp==3) % fixed temp
        a=(h.tend-h.tair)/(h.tbas-h.tair);
        h.etemp(1:h.nxi)=a*sinh(m*x)+sinh(m*(l-x));
        h.etemp(1:h.nxi)=h.etemp(1:h.nxi)/sinh(m*l);
        h.qf=M*(cosh(m*l)-a)/sinh(m*l);
        
        % flux
        h.eflux=h.cond.*m.*(h.tbas-h.tair);
        h.eflux=h.eflux*(cosh(m.*(l-x))-a.*cosh(m.*x));
        h.eflux=h.eflux/sinh(m.*l);
        
    elseif (h.bctp==4) % infinite fin
        h.etemp=exp(-m*x);
        h.qf=M;
        h.eflux=h.cond.*m.*(h.tbas-h.tair);
        h.eflux=h.eflux*exp(-m.*x);
    end
    
    % Get theta to temp
    h.etemp(1:h.nxi)=h.etemp(1:h.nxi)*(h.tbas-h.tair);
    h.etemp(1:h.nxi)=h.etemp(1:h.nxi)+h.tair;

    % Get surf plot
    for i=1:h.neta
        h.ez(i,1:h.nxi)=h.etemp(1:h.nxi);
    end
    
    % get effectivnes, base flux
    h.ef=h.qf/(h.conv*h.area*(h.tbas-h.tair));   
    
    % Get efficienc
    h.nf=h.qf/(h.conv*h.perim*l*(h.tbas-h.tair));
    
end

% *************************************************************************
%  PLOT UPDATE - Generate new plot
% *************************************************************************
function h=update_plot(h)

    % Select color axes
    axes(h.color_fig);    
    

%     h.gmax(3)=max([get(h.sliders(2,1),'Max'),get(h.sliders(2,4),'Max')]); % templimits
%     caxis(h.color_fig,[0 h.gmax(3)]);
    
    % Update Data
    
    hold on
    set(h.plot1,'XData',h.ex(:,:));
    set(h.plot1,'YData',h.ey(:,:));
    set(h.plot1,'ZData',h.ez(:,:));     
    colormap hot
    minl=min(min(h.ez));
    maxl=max(max(h.ez));
    caxis([minl,maxl]);
    hold off

    % plot bounds
%     h.gmax(1)=sum([get(h.sliders(4,1),'Max'),get(h.sliders(4,2),'Max'),get(h.sliders(4,3),'Max')]); % templimits
%     h.gmax(2)=max([get(h.sliders(1,1),'Max'),get(h.sliders(1,2),'Max'),get(h.sliders(1,3),'Max'),get(h.sliders(1,4),'Max')]); % templimits
    
    % Set limits
%     xlim(h.color_fig,[0 h.gmax(1)]);
%     ylim(h.color_fig,[-0.5*h.gmax(2) 0.5*h.gmax(2)]);  
%     colormap(h.color_fig,'hot');
%     set(h.plot1,'CLim',[0 h.gmax(3)]);
%     set(h.plot2,'XLim',[0 h.gmax(1)]);
%     set(h.plot2,'YLim',[-0.5*h.gmax(2) 0.5*h.gmax(2)]);  
%     set(h.plot2,'CLim',[0 h.gmax(3)]);    set(h.plot1,'XLim',[0 h.gmax(1)]);
%     set(h.plot3,'XLim',[0 h.gmax(1)]);
%     set(h.plot3,'YLim',[-0.5*h.gmax(2) 0.5*h.gmax(2)]);  
%     set(h.plot3,'CLim',[0 h.gmax(3)]);    
%     
%     caxis([0 h.gmax(3)]);
    view(0,90);
    
    % Plot cross-section
    % **********************************
    axes(h.data_fig1);   
    grid on
    
    % select left side
    yyaxis left
    plot( h.ex(1,:),h.etemp(:) );
    ylabel('Temperature (K)');
    xlabel('Fin Axial Location (m)');
    
    % select right side
    yyaxis right
    plot( h.ex(1,:),h.eflux(:) );    
    ylabel('Flux (W/m^2)');
    
    % Plot Bar Plot (base heat flux and effectivness)
    % **********************************
    axes(h.bar_plot1);       
    grid on
    bar(h.qf);
    ylabel('Base Heat Flow (W)')
    
    axes(h.bar_plot2);          
    grid on
    bar(h.ef);
    ylabel('Fin Effectivness')

    % Pie Plot (Fin effiiceness)
    % **********************************
    axes(h.pie_plot);      
    pie([h.nf,(1.0-h.nf)])
    title('Fin Efficiency')

    % Cross-section
    % **********************************
    axes(h.data_fig2);
    fill(h.vertx,h.verty,'k');
    grid on
    title('Cross-Section')
    xlabel('Length (m)')
    ylabel('Height (m)')
    maxl=max([h.vertx,h.verty]);
    minl=min([h.vertx,h.verty]);
    xlim([minl,maxl]);
    ylim([minl,maxl]);
    
end

% *************************************************************************
%  Contact Resistance 
% *************************************************************************
function h=update_cross_section(hObject, eventdata, h,idx,dtype)
    
    % Update edit box or slider
    if (idx>0)
        h=update_entry(hObject, eventdata, h, idx, dtype);
    end
    
    % update cross section (0=aspect ratio, 1=area)
    if (idx==2) %aspect ratio
         h.arat=get(h.sliders(2),'Value');
    elseif (idx==-1) % button select
         h.atyp=dtype; % button update (1=tri,2=rect,3=ellipse)
    else % Area update
         h.area=get(h.sliders(1),'Value');
    end
    
    % Redefine cross section
    h=get_cross_section(h);
    
    % call new mesh on local segment
    h=update_mesh(h);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% Update cross section
function h=get_cross_section(h)
    
    % Get diameter and perimeter
    if (h.atyp==1) % triangle
        b=sqrt(2.0*h.area/h.arat);
        hb=h.arat*b;
        h.vertx=[0,b,b/2.0];
        h.verty=[0,0,hb];
        if (hb>b)
            h.diam=hb;
        else
            h.diam=b;
        end
        h.perim=b+2.0*sqrt(0.25*b*b+hb*hb);
    elseif (h.atyp==2) % Rectangle
        a=sqrt(h.arat*h.area);
        b=h.area/a;
        h.perim=2.0*(a+b);
        if (a>b)
            h.diam=a;
        else
            h.diam=b;
        end
        h.vertx=[0,b,b,0];
        h.verty=[0,0,a,a];
    else % Ellipse
        b=sqrt(h.area/(pi*h.arat));
        a=h.arat*b;
        h.perim=(a-b)*(a-b)/((a+b)*(a+b));
        h.perim=pi*(a+b)*(1.0+(3*h.perim)/(10.0+sqrt(4.0-3.0*h.perim)));
        if (a>b)
            h.diam=2.0*a;
        else
            h.diam=2.0*b;
        end
        h.vertx=[linspace(-a,a,40),linspace(a,-a,40)];
        h.verty(1:40)=(b./a).*sqrt(a.*a-h.vertx(1:40).*h.vertx(1:40));
        h.verty(41:80)=-(b./a).*sqrt(a.*a-h.vertx(41:80).*h.vertx(41:80));
    end

end

% *************************************************************************
%  diameter Definition Callbacks
% *************************************************************************
function h=update_leng(hObject, eventdata,h,dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.leng=get(h.sliders(3),'Value');
    else % live text
        h.leng=str2num(get(h.lives(3),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, 3, dtype);
    
    % call new mesh on local segment
    h=update_mesh(h);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
    % Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Conductivity Values
% *************************************************************************
function h=update_conductivity(hObject, eventdata, h, idx, dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.cond=get(h.sliders(idx),'Value');
    elseif (dtype==2) %list box
        h.cond=klookup(get(h.lists(idx),'Value'),h);
    else % live text
        h.cond=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Convection Coefficient
% *************************************************************************
function h=update_conv(hObject, eventdata, h, idx, dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.conv=get(h.sliders(idx),'Value');
    else % live text
        h.conv=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
% Air Temperature
% *************************************************************************
function h=update_temp_air(hObject, eventdata, h, idx, dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.tair=get(h.sliders(idx),'Value');
    else % live text
        h.tair=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Base Temperature 
% *************************************************************************
function h=update_temp_base(hObject, eventdata, h, idx, dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.tbas=get(h.sliders(idx),'Value');
    else % live text
        h.tbas=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Base Temperature 
% *************************************************************************
function h=update_temp_end(hObject, eventdata, h, idx, dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.tend=get(h.sliders(idx),'Value');
    else % live text
        h.tend=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% Update Boundary condition
function h=update_bc(hObject, eventdata, h,dtype,idx)
    
   % Set bc type
   h.bctp=idx;
    
      % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
% *************************************************************************
%  Utility Functions
% *************************************************************************
% *************************************************************************

% *************************************************************************
% Generate conductiivty data
% *************************************************************************
function value=klookup(idx,h)

    % if custom, display original value
    if (idx>=length(h.karray))
        value=h.invars(2);
    else % get value from array
        value=h.karray(idx);
    end
    
end

% % *************************************************************************
% %  Export to CSV
% % *************************************************************************
% function export_to_csv_Callback(hObject, eventdata, h)
% end
% 
% % *************************************************************************
% %  Export to workspace
% % *************************************************************************
% function export_to_workspace_Callback(hObject, eventdata, h)
% end

% % *************************************************************************
% %  Take Screenshot
% % *************************************************************************
% function screenshot_button_Callback(hObject, eventdata, h)
% end

% *************************************************************************
%  Reset Data
% *************************************************************************
function reset_data_Callback(hObject, eventdata, h)
%     delete(h.plot1);
%     delete(h.plot2)
%     delete(h.plot3)
	plot_data_init(hObject,h);
end

% *************************************************************************
%  Make sure live text - slider - and popups all work in sync
% *************************************************************************
function h=update_entry(hObject, eventdata, h, idx, dtype)
    
    % if slider was changed
    if (dtype==0) % update live text, maybe popup
       
        % get value
        val=get(h.sliders(idx),'Value');
        
        % update live text
        set(h.lives(idx),'String',num2str2(val));
        
        % if thermal conducivity, set to custom
        if (idx==4) 
            set(h.lists(idx),'Value',12);
        end
        
    elseif (dtype==1) % if live was changed, update slider, maybe popup
        
        % get value
        val=str2num(get(h.lives(idx),'String'));
        
        % reset slider range if out of bounds
        h=slider_update(val,idx,h);
        
        % if thermal conducitivty, set to custom
        if (idx==4)
            set(h.lists(idx),'Value',12);
        end
        
    else % popup changed
        
        % get value
        val=klookup(get(h.lists(idx),'Value'),h);
        
        % reset slider range if out of bounds
        h=slider_update(val,idx,h);
        
        % fill live test field
        set(h.lives(idx),'String',num2str2(val));
        
    end
        
end

% *************************************************************************
%  Used in update_entry, common routine for updating sliders
% *************************************************************************
function h=slider_update(val,idx,h)
       
    % Get current rangers
    max_slider=get(h.sliders(idx),'Max');
    min_slider=get(h.sliders(idx),'Min');
    
    % If new value is out of bounds
    if (val>max_slider) % new value too big
        set(h.sliders(idx),'Max',val*h.mult_limit(idx)); 
        set(h.sliders(idx),'Value',val);
        set(h.sliders(idx),'Min',val/h.mult_limit(idx));
        
    elseif (val<min_slider) % New value too small
        set(h.sliders(idx),'Min',val/h.mult_limit(idx));
        set(h.sliders(idx),'Value',val);
        set(h.sliders(idx),'Max',val*h.mult_limit(idx));
        
    else % New value jsut right
        set(h.sliders(idx),'Value',val);
    end 

end

% *************************************************************************
%  Live Text - proper precision
% *************************************************************************
function str=num2str2(val)

    % Determine precision type
    if (val>10.0) % no decimals
        str=num2str(val,'%.0f');
    elseif (val>1.0) % 1 deciaml
        str=num2str(val,'%.1f');
    else % 8 preciision
        str=num2str(val,'%.3g');
    end
        
end

% maight be able to delete, returns to command line
function varargout = plot_basic_OutputFcn(hObject, eventdata, h) 
varargout{1} = 0;
end
