function varargout = plot_basic(varargin)

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @plot_basic_OpeningFcn, ...
                   'gui_OutputFcn',  @plot_basic_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

end

% *************************************************************************
%  GUI Initialization - runs once!
% *************************************************************************
function plot_basic_OpeningFcn(hObject, eventdata, h, varargin)
     
    % Clear matlab terminal
%     clc
        
    % Get arugments from live script, implied order:
    % diameter,k,length,temp left, temp right,resistance1,resistance2
    h.invars=cell2mat(varargin);
    
    % Check for number of arugments
    if(length(h.invars)~=7)
       h.invars(1:7)=[0.5,1,1,1.2,1000.0,300.0,2.0];
       % volume,shape,material,h,temp solid,temp air,time
    end
   
    plot_data_init(hObject,h);
        
end


% *************************************************************************
%  Data Initalization - Called at start and reset
% *************************************************************************
function plot_data_init(hObject,h)
    
    % Defined Global Constants
    h.nxi=100;
    nxi=h.nxi;
    
    % Plot Indexing
    % 1 = Shape Index (1-3)
    % 2 = Shape Volume (m^3)
    % 3 = Heat Transfer Coeff. (W/(k*m^2))
    % 4 = Material Type (1-N)
    % 5 = Density (kg/m^3)
    % 6 = Heat Capacity (J/K)
    % 7 = Solid Mass Temp (Kelvin)
    % 8 = Air Temp (Kelvin)
    % 9 = Max Time (sec)
    
    % Set multipliers, values greater than 1 only
    h.mult_limit(1:9)=[0,2,2,2,2,2,2,2,2]; % set diameter limits
    
    % Initialize Material (Heat Capacity J/(kg*K), Density kg/m^3)
    h.c=[2000.0, 1760.0, 880.0, 2108.0, 502.4, 385.0, 630.0, 14304.0];
    h.dens=[1.0, 740.0, 2300.0, 917.0, 8050.0, 2700.0, 900.0, 129.0, 19300.0, 8960.0, 3520.0, 19600.0 ];
    h.karray=[0.02, 0.05, 0.1, 1.5, 2.22, 50.2, 237, 310, 401, 1000, 10000];
    
    % shape datya
    h.volume=h.invars(1); % volume
    h.shape=h.invars(2); % shape type, set to triangular prism
    
    % GEt surface area
    if (h.shape==1)
        r=sqrt(3.0*h.volume);
        h.area=sqrt(3.0)*r*r;
    elseif (h.shape==2) %cube
        r=(h.volume).^(1.0/3.0);
        h.area=6.0*r*r;
    else % sphere
        r=(0.75.*h.volume./3.1415926).^(1.0/3.0);
        h.area=4.0.*3.1415926.*r.*r;
    end 
    
    % update Other Parameters
    h.material=h.invars(3); % material index
    h.hcoef=h.invars(4); % heat transfer coefficient
    
    % Initialize material prop data
    h.k=h.karray(h.material);
    h.ccap=h.c(h.material);
    h.rho=h.dens(h.material);
    
    % Initialize Sim Data
    h.tmass=h.invars(5); % mass temp initial 
    h.tair=h.invars(6); % air temp
    h.mtime=h.invars(7); % max time

        
    % Initialize mesh and surf
    h.ex=zeros(nxi,1);
    
    % Initialize output vars
    h.xtemp=zeros(nxi,1);
    h.xtheta=zeros(nxi,1);
    h.xflux=zeros(nxi,1);
    h.xfourier=zeros(nxi,1);
    h.fo_max=0.0;
    h.biot=0.0;
    
    % Define handle array (row on gui:column on gui)
    % note sparse in element rows!
    h.sliders=[0,h.volume_slider,h.h_slider,h.k_slider,h.rho_slider,h.c_slider,...
               h.t0_slider,h.tair_slider,h.maxtime_slider];
           
    h.lives=[0,h.volume_live,h.h_live,h.k_live,h.rho_live,h.c_live,...
               h.t0_live,h.tair_live,h.maxtime_live]; 
    
    h.lists=[h.material_list];
    
%     set(h.lists(4),'Value',12);
    
    % get initialization matrix (local scope)
    init(1:9)=[h.invars(2),h.invars(1),h.invars(3),h.k,...
               h.rho,h.ccap,h.invars(5:7)];
    for j=2:9
        set(h.sliders(j),'Max',init(j)*h.mult_limit(j));
        set(h.sliders(j),'Min',init(j)/h.mult_limit(j));
        set(h.sliders(j),'Value',init(j));
        set(h.lives(j),'String',num2str(init(j)));
    end
    
    % Initialize material 
    set(h.material_list,'Value',h.material);
    
    % Set selection values
%     h.select=[h.ar_set,h.bc_set];    
    
    % Intiailize plots
%     axes(h.data1);
%     hold on
%     h.plot1=plot(h.ex(:),h.xtemp(:));
%     hold off
    
    % Get new data in ez1:3
    % Should implicity update live fields
    h=update_data(h);
    
    % Generate new plots
    h=update_plot(h);
        
    % Choose default command line output for plot_basic
    h.output = hObject;
    
    % Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  DATA UPDATE - Generate new data piecmeal
% *************************************************************************
function h=update_data(h)
    
    % Get Lc
    lc=(h.volume./h.area);
    
    % Update Time Array
    h.ex=linspace(0.0,h.mtime,h.nxi);
    
    % Get Fourier number
    alpha=h.k/((h.rho).*(h.ccap));
    h.xfourier=h.ex.*alpha./(lc.^2.0);
    h.fo_max=max(abs(h.xfourier));
    
    % Get tau constant
    tau1=h.rho*h.volume*h.ccap/(h.hcoef*h.area);
    
    % Biot number
    h.biot=h.hcoef*lc/h.k;
    
    % Get temp flux
    h.xtheta=exp(-h.biot.*h.xfourier);
    h.xtemp=h.xtheta.*(h.tmass-h.tair)+h.tair;
    h.xflux=h.xtheta(1).*h.rho.*h.volume.*h.ccap;
    h.xflux=h.xflux.*(1.0-exp(-h.ex./tau1));
    
end

% *************************************************************************
%  PLOT UPDATE - Generate new plot
% *************************************************************************
function h=update_plot(h)

    % Select temp/theta plot (vs flux/fourier plot)
    axes(h.data1);   
    grid on
    
    % select left side
    yyaxis left
    plot( h.ex(:),h.xtemp(:) );
    ylabel('Temperature (K)');
    
    % select right side
    yyaxis right
    plot( h.ex(:),h.xtheta(:) );    
    ylabel('Theta (arb.)');
    
    % Plot Bar Plot (base heat flux and effectivness)
    % **********************************
    axes(h.data2);       
    grid on

    % select left side
    yyaxis left
    plot( h.ex(:),h.xflux(:) );
    ylabel('Flux (Joules)');
    xlabel('Time (s)');
    
    % select right side
    yyaxis right
    plot( h.ex(:),h.xfourier(:) );    
    ylabel('Fourier Number (arb.)');
    
    % Select first bar plot (max/min fourier)
    axes(h.fourier_plot);
    bar([h.xfourier(end)]);
    grid on
    ylabel('Fourier Numbers (arb.)')
    
    % Plot biot number
    axes(h.biot_plot);          
    bar(h.biot);
    grid on
    ylabel('Biot Number (arb.)')
        
end

% *************************************************************************
% Shape Selection
% *************************************************************************
function h=update_shape(hObject, eventdata, h, idx, dtype)
    
    % Update Shape type
    h.shape=dtype;
    h=update_volume(hObject, eventdata, h, idx, -1);
    
end

% *************************************************************************
%  Volume
% *************************************************************************
function h=update_volume(hObject, eventdata, h, idx, dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.volume=get(h.sliders(idx),'Value');
    elseif (dtype==1)% live text
        h.volume=str2num(get(h.lives(idx),'String')); 
    end
    
    % GEt surface area
    if (h.shape==1)
        r=sqrt(3.0*h.volume);
        h.area=sqrt(3.0)*r*r;
    elseif (h.shape==2) %cube
        r=(h.volume).^(1.0/3.0);
        h.area=6.0*r*r;
    else % sphere
        r=(0.75.*h.volume./3.1415926).^(1.0/3.0);
        h.area=4.0.*3.1415926.*r.*r;
    end 
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Convection Coefficient
% *************************************************************************
function h=update_conv(hObject, eventdata, h, idx, dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.hcoef=get(h.sliders(idx),'Value');
    else % live text
        h.hcoef=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Material Selection
% *************************************************************************
function h=material_list(hObject, eventdata, h, idx, dtype)
       
    % Update h.metarial
    h.material=get(h.material_list,'Value');
    
    % update Values
    if (h.material<11)
        
        % Material set
        h.ccap=h.c(h.material);
        h.rho=h.dens(h.material);
        h.k=h.karray(h.material);
                   
        % update live boxes
        set(h.lives(4),'Value',h.karray(h.material));
        set(h.lives(4),'String',h.karray(h.material));
        set(h.lives(5),'Value',h.dens(h.material));
        set(h.lives(5),'String',h.dens(h.material));
        set(h.lives(6),'Value',h.c(h.material));
        set(h.lives(6),'String',h.c(h.material));
        
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, 5, 1);
    h=update_entry(hObject, eventdata, h, 6, 1);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Density
% *************************************************************************
function h=update_rho(hObject, eventdata, h, idx, dtype)
       
    % Update h.metarial
    if (dtype==0) % slider
        h.rho=get(h.sliders(idx),'Value');
    else % live text
        h.rho=str2num(get(h.lives(idx),'String')); 
    end
    
    % update material list
    set(h.material_list,'Value',11.0);
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Heat Capacity
% *************************************************************************
function h=update_ccap(hObject, eventdata, h, idx, dtype)
       
    % Update h.metarial
    if (dtype==0) % slider
        h.ccap=get(h.sliders(idx),'Value');
    else % live text
        h.ccap=str2num(get(h.lives(idx),'String')); 
    end
    
    % update material list
    set(h.material_list,'Value',11.0);
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Thermal Conductivity
% *************************************************************************
function h=update_karray(hObject, eventdata, h, idx, dtype)
       
    % Update h.metarial
    if (dtype==0) % slider
        h.k=get(h.sliders(idx),'Value');
    else % live text
        h.k=str2num(get(h.lives(idx),'String')); 
    end
    
    % update material list to custom
    set(h.material_list,'Value',11.0);
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Initial Object Temperature 
% *************************************************************************
function h=update_tmass(hObject, eventdata, h, idx, dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.tmass=get(h.sliders(idx),'Value');
    else % live text
        h.tmass=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
% Air Temperature
% *************************************************************************
function h=update_tair(hObject, eventdata, h, idx, dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.tair=get(h.sliders(idx),'Value');
    else % live text
        h.tair=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
% Max Time (sec)
% *************************************************************************
function h=update_mtime(hObject, eventdata, h, idx, dtype)
    
    % update diamater & area
    if (dtype==0) % slider
        h.mtime=get(h.sliders(idx),'Value');
    else % live text
        h.mtime=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
% *************************************************************************
%  Utility Functions
% *************************************************************************
% *************************************************************************

% *************************************************************************
%  Reset Data
% *************************************************************************
function reset_data_Callback(hObject, eventdata, h)
%     delete(h.plot1);
%     delete(h.plot2)
%     delete(h.plot3)
	plot_data_init(hObject,h);
end

% *************************************************************************
%  Make sure live text - slider - and popups all work in sync
% *************************************************************************
function h=update_entry(hObject, eventdata, h, idx, dtype)
    
    % if slider was changed
    if (dtype==0) % update live text, maybe popup
       
        % get value
        val=get(h.sliders(idx),'Value');
        
        % update live text
        set(h.lives(idx),'String',num2str2(val));
        
        % if thermal conducivity, set to custom
%         if (idx==4) 
%             set(h.lists(idx),'Value',12);
%         end
        
    elseif (dtype==1) % if live was changed, update slider, maybe popup
        
        % get value
        val=str2num(get(h.lives(idx),'String'));
        
        % reset slider range if out of bounds
        h=slider_update(val,idx,h);
        
        % if thermal conducitivty, set to custom
%         if (idx==4)
%             set(h.lists(idx),'Value',12);
%         end
        
    else % popup changed
        
%         % get value
%         val=klookup(get(h.lists(idx),'Value'),h);
%         
%         % reset slider range if out of bounds
%         h=slider_update(val,idx,h);
%         
%         % fill live test field
%         set(h.lives(idx),'String',num2str2(val));
%         
    end
        
end

% *************************************************************************
%  Used in update_entry, common routine for updating sliders
% *************************************************************************
function h=slider_update(val,idx,h)
       
    % Get current rangers
    max_slider=get(h.sliders(idx),'Max');
    min_slider=get(h.sliders(idx),'Min');
    
    % If new value is out of bounds
    if (val>max_slider) % new value too big
        set(h.sliders(idx),'Max',val*h.mult_limit(idx)); 
        set(h.sliders(idx),'Value',val);
        set(h.sliders(idx),'Min',val/h.mult_limit(idx));
        
    elseif (val<min_slider) % New value too small
        set(h.sliders(idx),'Min',val/h.mult_limit(idx));
        set(h.sliders(idx),'Value',val);
        set(h.sliders(idx),'Max',val*h.mult_limit(idx));
        
    else % New value jsut right
        set(h.sliders(idx),'Value',val);
    end 

end

% *************************************************************************
%  Live Text - proper precision
% *************************************************************************
function str=num2str2(val)

    % Determine precision type
    if (val>10.0) % no decimals
        str=num2str(val,'%.0f');
    elseif (val>1.0) % 1 deciaml
        str=num2str(val,'%.1f');
    else % 8 preciision
        str=num2str(val,'%.3g');
    end
        
end

% maight be able to delete, returns to command line
function varargout = plot_basic_OutputFcn(hObject, eventdata, h) 
varargout{1} = 0;
end
