function varargout = plot_basic(varargin)

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @plot_basic_OpeningFcn, ...
                   'gui_OutputFcn',  @plot_basic_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

% *************************************************************************
%  GUI Initialization - runs once!
% *************************************************************************
function plot_basic_OpeningFcn(hObject, eventdata, h, varargin)
     
    % Clear matlab terminal
    clc
        
    % Get arugments from live script, implied order:
    % diameter,k,length,temp left, temp right,resistance1,resistance2
    h.invars=cell2mat(varargin);
    
    % Check for number of arugments
    if(length(h.invars)~=6)
       disp('Need exactly 6 arguments')
       disp('Using hard-coded vars!')
       h.invars(1:6)=[1.0, ...         % Object Diam (cm)
                      348.0, ...       % Surface Temp (K)
                      2, ...           % Cross-Section Index
                      298.0, ...       % Air Temp (K)
                      1, ...           % Set Fluid type
                      1500.0 ];        % fluid speed (cm/s)
    end
   
    plot_data_init(hObject,h);
    
end

end

% *************************************************************************
%  Data Initalization - Called at start and reset
% *************************************************************************
function plot_data_init(hObject,h)
    
    % Initialize figures (for showing dimension diagrams)
    axes(h.axes9); mtImg = imread('figs/1cylinder.png');
    image(mtImg); axis off; axis image;
    
    axes(h.axes10); mtImg = imread('figs/2square.png');
    image(mtImg); axis off; axis image;
    
    axes(h.axes11); mtImg = imread('figs/3inclined_square_diamond.png');
    image(mtImg); axis off; axis image;
    
    axes(h.axes12); mtImg = imread('figs/4thin_plate.png');
    image(mtImg); axis off; axis image;
    
    

    % Initialize Plot Data
    % ***********************************************************
    h.nxi=0;   % Current number of points
    h.nmax=50; % maximum number of points
    h.q_array(1:h.nmax)=0.0; % Array of output q
    h.h_array(1:h.nmax)=0.0; % Array of output h
    h.re_array(1:h.nmax)=0.0; % Array of output re
    h.nud_array(1:h.nmax)=0.0; % Array of output nu
    h.case_num(1:h.nmax)=1:h.nmax;
    
    % Initialize Scalar Parameters
    % **********************************************************
    h.diam=h.invars(1)/100.0; % Converted from cm to m
    h.ts=h.invars(2);         % Surface Temp (K)
    h.tinf=h.invars(4);       % Fluid Inlet Temp (K)
    h.tfilm=0.5*(h.ts+h.tinf); % Tfilm (K)
    h.vel=h.invars(6)/100.0;  % Fluid Velocity from cm/s to m/s
    
    % Initialize Fluid Data
    % ***********************************************************
        
    % Set Maximum Fluids
    h.nfluids=4;
    
    % Fluid Index 
    h.idx_fluid=h.invars(5);
    
    % initialize fluid properties
    vals=get_data(h.tfilm,h.idx_fluid);
    h.kf=vals(1);
    h.nu=vals(2);
    h.pr=vals(3);
    
    % Correlation Data
    % ********************************************************** 
    
    % cm Matricie
    h.nre=[1,5,1,1];
    h.nshapes=length(h.nre);    
    
    % Flat plate - 1
    h.re_ref(1:h.nre(1),1)=[8.0e4];
    h.c_mat(1:h.nre(1),1)=[0.191];
    h.m_mat(1:h.nre(1),1)=[0.667];
    
    % Circular Cylinder - 2
    h.re_ref(1:h.nre(2),2)=[4.0,40.0,4.0e3,4.0e4,4.0e5];
    h.c_mat(1:h.nre(2),2)=[0.989,0.911,0.683,0.193,0.027];
    h.m_mat(1:h.nre(2),2)=[0.330,0.385,0.466,0.618,0.805];
    
    % Square Shape - 3
    h.re_ref(1:h.nre(3),3)=[6.0e4];    
    h.c_mat(1:h.nre(3),3)=[0.158];
    h.m_mat(1:h.nre(3),3)=[0.66];
    
    % Diamond  - 4
    h.re_ref(1:h.nre(4),4)=[6.0e4];
    h.c_mat(1:h.nre(4),4)=[0.304];
    h.m_mat(1:h.nre(4),4)=[0.59];
    
    % Cross-Section Initial Assignment
    h.ishape=h.invars(3);   % Cross-Section Index
    h.ire=1;                % Re index
    h.c=h.c_mat(h.ire,h.ishape);
    h.m=h.m_mat(h.ire,h.ishape);
        
    % Initialize Text output vars
    % **********************************************************
    h.tfilm=0.0;
    h.p=0.0;
    h.re=0.0;
    h.nud=0.0;
    h.hbar=0.0;
    h.q=0.0;
    
    % Set multipliers, values greater than 1 only
    h.mult_limit(1:7)=[2,2,2,2,2,2,2]; % set diameter limits
    
    % Define handle arrays
    % **********************************************************
    h.sliders=[h.diam_slider,h.ts_slider,h.tinf_slider,h.nu_slider...
               h.kf_slider,h.pr_slider,h.vel_slider];
    
    h.lives=[h.diam_live,h.ts_live,h.tinf_live,h.nu_live...
               h.kf_live,h.pr_live,h.vel_live];
    
    % Initialization of values
    % **********************************************************
    
    % Fluid Material
    set(h.fluid_list,'Value',h.idx_fluid)
    
    % Set Buttons
    buttons=[h.button_xsec_1,h.button_xsec_2,h.button_xsec_3,...
             h.button_xsec_4];
    set(h.button_xsec_1,'Value',0);
    set(h.button_xsec_2,'Value',0);
    set(h.button_xsec_3,'Value',0);
    set(h.button_xsec_4,'Value',0);   
    set(buttons(h.ishape),'Value',1);
    
    % Init Array
    init(1:7)=[h.diam*100.0,h.ts,h.tinf,h.nu,h.kf,h.pr,h.vel*100.0];
    
    % Set Sliders and Lives
    for j=1:7
        set(h.sliders(j),'Max',init(j)*h.mult_limit(j));
        set(h.sliders(j),'Min',init(j)/h.mult_limit(j));
        set(h.sliders(j),'Value',init(j));
        set(h.lives(j),'String',num2str(init(j)));
    end
    
    % Get data
    % **********************************************************
    h=update_data(h);
    
    % Initialize Plots
    % **********************************************************
    
    % Re, Nu Plots
    axes(h.ReNu_plot);  
    cla(h.ReNu_plot);
    cla reset
    hold on
    grid on
    title('Evaluated Results')
    yyaxis left
    h.plot_re=plot(h.case_num(1:h.nmax),h.re_array(1:h.nmax),'-');
    ylabel('Reynolds Number');    
    yyaxis right
    h.plot_nu=plot(h.case_num(1:h.nmax),h.nud_array(1:h.nmax),'-');
    ylabel('Nusselt Number');
    xlim([0,55])
    hold off
    
    % Plot qh Axes
    axes(h.qh_plot);
    cla(h.qh_plot);
    cla reset
    hold on
    grid on
    xlabel('Case Number')
    yyaxis left
    h.plot_h=plot(h.case_num(1:h.nmax),h.h_array(1:h.nmax),'-');
    ylabel([{'Heat Transfer Coeff.'};{'(W/m^2/K)'}]);   
    yyaxis right
    h.plot_q=plot(h.case_num(1:h.nmax),h.q_array(1:h.nmax),'-');
    ylabel('Heat Transfer (W/m)');    
    xlim([0,55])
    hold off
    
    % Update Plot Data - Maybe redundant
    % **********************************************************
    h=update_plot(h);
    
    % Choose default command line output for plot_basic
    % **********************************************************
    h.output = hObject;
    
    % Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  DATA UPDATE - Generate new data piecmeal
% *************************************************************************
function h=update_data(h)
    
    % Get Perimeter of shape
    if (h.ishape==1) % Flat Plate
        h.p=2.0*h.diam;
    elseif (h.ishape==2) % Circular Cylinder
        h.p=pi*h.diam; %circumference
    elseif(h.ishape>=3) % Square or diamond Prism
        h.p=4.0*h.diam;
    end

    % Get effective diameter
    d=h.diam;
    if (h.ishape==4)
        d=d*sqrt(2);
    end
    
    % Update Coefficients
    h.re=h.vel*d/h.nu;
    
    % Reynolds Number
    h.ire=1;
    i=1;
    while  (i<=h.nre(h.ishape) && h.re>h.re_ref(i,h.ishape))
        i=i+1;
    end
    h.ire=i;
    
    % C/M from lookup table
    h.c=h.c_mat(h.ire,h.ishape);
    h.m=h.m_mat(h.ire,h.ishape);
    
    % Nusselt Number
    h.nud=h.c*(h.re^h.m)*(h.pr^(1.0/3.0));
    
    % H Value
    h.hbar=h.kf*h.nud/h.diam;
    
    % Q Value
    h.q=h.hbar*h.p*(h.ts-h.tinf);
    
    % Shift Left
    h.re_array(1:h.nmax-1)=h.re_array(2:h.nmax);
    h.nud_array(1:h.nmax-1)=h.nud_array(2:h.nmax);
    h.h_array(1:h.nmax-1)=h.h_array(2:h.nmax);
    h.q_array(1:h.nmax-1)=h.q_array(2:h.nmax);
    
    % If First pass, make all the initial value (straight line)
    if (h.nxi==0)
        h.re_array(1:h.nmax-1)=h.re;
        h.nud_array(1:h.nmax-1)=h.nud;
        h.h_array(1:h.nmax-1)=h.hbar;
        h.q_array(1:h.nmax-1)=h.q;        
    end
    
    % Move nxi to last spot
    h.nxi=h.nmax;
    
    % Add New point
    h.re_array(h.nxi)=h.re;
    h.nud_array(h.nxi)=h.nud;
    h.h_array(h.nxi)=h.hbar;
    h.q_array(h.nxi)=h.q;
    
end

% *************************************************************************
%  PLOT UPDATE - Update plot Data
% *************************************************************************
function h=update_plot(h)
    
    % Update Re_Nu Data
    axes(h.ReNu_plot);  
    hold on
    set(h.plot_re,'YData',h.re_array(1:h.nmax));
    set(h.plot_nu,'YData',h.nud_array(1:h.nmax));   
    hold off
    
    % Plot qe Axes
    axes(h.qh_plot);
    hold on
    set(h.plot_h,'YData', h.h_array(1:h.nmax));
    set(h.plot_q,'YData', h.q_array(1:h.nmax));     
    hold off
    
    % Update text displays
    set(h.re_disp,'String',num2str(h.re,'%10.2f'));
    set(h.nu_disp,'String',num2str(h.nud,'%10.2f'));
    set(h.h_disp,'String',num2str(h.hbar,'%10.4g'));
    set(h.q_disp,'String',num2str(h.q,'%10.4g'));
    
end

% *************************************************************************
%  Update shape diameter
% *************************************************************************
function h=diam_update(hObject, eventdata, h, dtype, idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.diam=get(h.sliders(idx),'Value');
    else % live text
        h.diam=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % Update unit to m
    h.diam=h.diam/100.0; 
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Surface Temp
% *************************************************************************
function h=ts_update(hObject, eventdata, h, dtype, idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.ts=get(h.sliders(idx),'Value');
    else % live text
        h.ts=str2num(get(h.lives(idx),'String')); 
    end
            
    % Get T Film
    h.tfilm=0.5*(h.ts+h.tinf);
    
    % initialize fluid properties
    vals=get_data(h.tfilm,h.idx_fluid);
    h.kf=vals(1);
    h.nu=vals(2);
    h.pr=vals(3);
    
    % Update Fluid Properties
    set(h.nu_slider,'Min',h.nu/h.mult_limit(4));
    set(h.kf_slider,'Min',h.kf/h.mult_limit(5));
    set(h.pr_slider,'Min',h.pr/h.mult_limit(6));        
    set(h.nu_slider,'Max',h.nu*h.mult_limit(4));
    set(h.kf_slider,'Max',h.kf*h.mult_limit(5));
    set(h.pr_slider,'Max',h.pr*h.mult_limit(6));   
    set(h.nu_slider,'Value',h.nu);
    set(h.kf_slider,'Value',h.kf);
    set(h.pr_slider,'Value',h.pr);
    set(h.nu_live,'String',num2str(h.nu,'%6.2g'));
    set(h.kf_live,'String',num2str(h.kf,'%6.2g'));
    set(h.pr_live,'String',num2str(h.pr,'%6.2f'));      
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update shape diameter
% *************************************************************************
function h=xsec_update(hObject, eventdata, h, idx)
    
    % Update Index
    h.ishape=idx;
        
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Fluid Temp
% *************************************************************************
function h=tinf_update(hObject, eventdata, h, dtype, idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.tinf=get(h.sliders(idx),'Value');
    else % live text
        h.tinf=str2num(get(h.lives(idx),'String')); 
    end
        
    % Get T Film
    h.tfilm=0.5*(h.ts+h.tinf);
    
    % initialize fluid properties
    vals=get_data(h.tfilm,h.idx_fluid);
    h.kf=vals(1);
    h.nu=vals(2);
    h.pr=vals(3);
    
    % Update Fluid Properties
    set(h.nu_slider,'Min',h.nu/h.mult_limit(4));
    set(h.kf_slider,'Min',h.kf/h.mult_limit(5));
    set(h.pr_slider,'Min',h.pr/h.mult_limit(6));        
    set(h.nu_slider,'Max',h.nu*h.mult_limit(4));
    set(h.kf_slider,'Max',h.kf*h.mult_limit(5));
    set(h.pr_slider,'Max',h.pr*h.mult_limit(6));   
    set(h.nu_slider,'Value',h.nu);
    set(h.kf_slider,'Value',h.kf);
    set(h.pr_slider,'Value',h.pr);
    set(h.nu_live,'String',num2str(h.nu,'%6.2g'));
    set(h.kf_live,'String',num2str(h.kf,'%6.2g'));
    set(h.pr_live,'String',num2str(h.pr,'%6.2f'));        
        
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Fluid Index
% *************************************************************************
function h=ftype_update(hObject, eventdata, h, dtype, idx)
    
    % Get Index
    h.idx_fluid=get(h.fluid_list,'Value');
    
    % If good index, then get prop
    if (h.idx_fluid<=h.nfluids)
        
        % Get T Film
        h.tfilm=0.5*(h.ts+h.tinf);
        
        % Retrieve new Fluid
        vals=get_data(h.tfilm,h.idx_fluid);
        h.kf=vals(1);
        h.nu=vals(2);
        h.pr=vals(3);
        
        % Update edit box or slider
        set(h.nu_slider,'Min',h.nu/h.mult_limit(4));
        set(h.kf_slider,'Min',h.kf/h.mult_limit(5));
        set(h.pr_slider,'Min',h.pr/h.mult_limit(6));        
        set(h.nu_slider,'Max',h.nu*h.mult_limit(4));
        set(h.kf_slider,'Max',h.kf*h.mult_limit(5));
        set(h.pr_slider,'Max',h.pr*h.mult_limit(6));   
        set(h.nu_slider,'Value',h.nu);
        set(h.kf_slider,'Value',h.kf);
        set(h.pr_slider,'Value',h.pr);
        set(h.nu_live,'String',num2str(h.nu,'%6.2g'));
        set(h.kf_live,'String',num2str(h.kf,'%6.2g'));
        set(h.pr_live,'String',num2str(h.pr,'%6.2f'));        
        
    end
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Kinematic Viscosity
% *************************************************************************
function h=nu_update(hObject, eventdata, h, dtype, idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.nu=get(h.sliders(idx),'Value');
    else % live text
        h.nu=str2num(get(h.lives(idx),'String')); 
    end
    
    % Set Fluid Type to custom
    set(h.fluid_list,'Value',h.nfluids+1);
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Thermal Conductivtiy
% *************************************************************************
function h=kf_update(hObject, eventdata, h, dtype, idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.kf=get(h.sliders(idx),'Value');
    else % live text
        h.kf=str2num(get(h.lives(idx),'String')); 
    end
    
    % Set Fluid Type to custom
    set(h.fluid_list,'Value',h.nfluids+1);
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Prandtl Number
% *************************************************************************
function h=pr_update(hObject, eventdata, h, dtype, idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.pr=get(h.sliders(idx),'Value');
    else % live text
        h.pr=str2num(get(h.lives(idx),'String')); 
    end
    
     % Set Fluid Type to custom
    set(h.fluid_list,'Value',h.nfluids+1);
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Fluid Speed
% *************************************************************************
function h=vel_update(hObject, eventdata, h, dtype, idx)
    
    % update
    if (dtype==0) % slider
        h.vel=get(h.sliders(idx),'Value');
    else % live text
        h.vel=str2num(get(h.lives(idx),'String')); 
    end
    
    % Adjust to m/s
    h.vel=h.vel/100.0;
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
    
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
% *************************************************************************
%  Utility Functions
% *************************************************************************
% *************************************************************************

% *************************************************************************
% Generate conductiivty data
% *************************************************************************

% *************************************************************************
%  Reset Data
% *************************************************************************
function reset_data_Callback(hObject, eventdata, h)
	plot_data_init(hObject,h);
end

% *************************************************************************
%  Reset  Plot
% *************************************************************************
function reset_plot_Callback(hObject, eventdata, h)

    h.re_array(1:h.nmax-1)=h.re;
    h.nud_array(1:h.nmax-1)=h.nud;
    h.h_array(1:h.nmax-1)=h.hbar;
    h.q_array(1:h.nmax-1)=h.q;      
    h=update_plot(h);

end

% *************************************************************************
%  Make sure live text - slider - and popups all work in sync
% *************************************************************************
function h=update_entry(hObject, eventdata, h, idx, dtype)
    
    % if slider was changed
    if (dtype==0) % update live text, maybe popup
       
        % get value
        val=get(h.sliders(idx),'Value');
        
        % update live text
        set(h.lives(idx),'String',num2str2(val));
        
        % if thermal conducivity, set to custom
%         if (idx==5) 
%             set(h.klists(idx),'Value',11);
%         end
        
    elseif (dtype==1) % if live was changed, update slider, maybe popup
        
        % get value
        val=str2num(get(h.lives(idx),'String'));
        
        % reset slider range if out of bounds
        h=slider_update(val,idx,h);
        
        % if thermal conducitivty, set to custom
%         if (idx==5)
%             set(h.klists(idx),'Value',11);
%         end
        
    else % popup changed
        
        % get value
%         if (idx==5) 
%             val=klookup(get(h.klists(idx),'Value'),h);
%         end
        
        % Reset slider range if out of bounds
        h=slider_update(val,idx,h);
        
        % fill live test field
        set(h.lives(idx),'String',num2str2(val));
        
    end
        
end

% *************************************************************************
%  Used in update_entry, common routine for updating sliders
% *************************************************************************
function h=slider_update(val,idx,h)
       
    % Get current rangers
    max_slider=get(h.sliders(idx),'Max');
    min_slider=get(h.sliders(idx),'Min');
    
    % If new value is out of bounds
    if (val>max_slider) % new value too big
        set(h.sliders(idx),'Max',val*h.mult_limit(idx)); 
        set(h.sliders(idx),'Value',val);
        set(h.sliders(idx),'Min',val/h.mult_limit(idx));
        
    elseif (val<min_slider) % New value too small
        set(h.sliders(idx),'Min',val/h.mult_limit(idx));
        set(h.sliders(idx),'Value',val);
        set(h.sliders(idx),'Max',val*h.mult_limit(idx));
        
    else % New value jsut right
        set(h.sliders(idx),'Value',val);
    end 

end

% *************************************************************************
%  Live Text - proper precision
% *************************************************************************
function str=num2str2(val)

    % Determine precision type
    if (val>10.0) % no decimals
        str=num2str(val,'%.0f');
    elseif (val>1.0) % 1 deciaml
        str=num2str(val,'%.1f');
    else % 8 preciision
        str=num2str(val,'%.3g');
    end
        
end

function varargout = plot_basic_OutputFcn(hObject, eventdata, h) 
varargout{1} = 0;
end


% *************************************************************************
% Load Material Property
% *************************************************************************
function vals=get_data(temp,fluid)

% fluid type
if (fluid==1) % Air
    
    % Store Values
   m=[123.15,0.01171,3.013e-6,0.7246;...
        173.15,0.01582,5.837e-6,0.7263;...
        223.15,0.01979,9.319e-6,0.744;...
        233.15,0.02057,1.008e-5,0.7436;...
        243.15,0.02134,1.087e-5,0.7425;...
        253.15,0.02211,1.169e-5,0.7408;...
        263.15,0.02288,1.252e-5,0.7387;...
        273.15,0.02364,1.338e-5,0.7362;...
        278.15,0.02401,1.382e-5,0.735;...
        283.15,0.02439,1.426e-5,0.7336;...
        288.15,0.02476,1.470e-5,0.7323;...
        293.15,0.02514,1.516e-5,0.7309;...
        298.15,0.02551,1.562e-5,0.7296;...
        303.15,0.02588,1.608e-5,0.7282;...
        308.15,0.02625,1.655e-5,0.7268;...
        313.15,0.02662,1.702e-5,0.7255;...
        318.15,0.02699,1.750e-5,0.7241;...
        323.15,0.02735,1.798e-5,0.7228;...
        333.15,0.02808,1.896e-5,0.7202;...
        343.15,0.02881,1.995e-5,0.7177;...
        353.15,0.02953,2.097e-5,0.7154;...
        363.15,0.03024,2.201e-5,0.7132;...
        373.15,0.03095,2.306e-5,0.7111;...
        393.15,0.03235,2.522e-5,0.7073;...
        413.15,0.03374,2.745e-5,0.7041;...
        433.15,0.03511,2.975e-5,0.7014;...
        453.15,0.03646,3.212e-5,0.6992;...
        473.15,0.03779,3.455e-5,0.6974;...
        523.15,0.04104,4.091e-5,0.6946;...
        573.15,0.04418,4.765e-5,0.6935;...
        623.15,0.04721,5.475e-5,0.6937;...
        673.15,0.05015,6.219e-5,0.6948;...
        723.15,0.05298,6.997e-5,0.6965;...
        773.15,0.05572,7.806e-5,0.6986;...
        873.15,0.06093,9.515e-5,0.7037;...
        973.15,0.06581,1.133e-4,0.7092;...
        1073.15,0.07037,1.326e-4,0.7149;...
        1173.15,0.07465,1.529e-4,0.7206;...
        1273.15,0.07868,1.741e-4,0.726;...
        1773.15,0.09599,2.922e-4,0.7478;...
        2273.15,0.11113,4.270e-4,0.7539];
    
elseif (fluid==2) % Co2
    
    m=[223.15,0.01051,4.70E-06,0.8019;...
        273.15,0.01456,7.00E-06,0.7661;...
        323.15,0.01858,9.71E-06,0.752;...
        373.15,0.02257,1.28E-05,0.7464;...
        423.15,0.02652,1.63E-05,0.7445;...
        473.15,0.03044,2.01E-05,0.7442;...
        573.15,0.03814,2.87E-05,0.745;...
        673.15,0.04565,3.84E-05,0.7458;...
        773.15,0.05293,4.92E-05,0.746;...
        1273.15,0.08491,1.16E-04,0.7455;...
        1773.15,0.10688,2.02E-04,0.7745;...
        2273.15,0.11522,3.10E-04,0.8815];

elseif (fluid==3) % Methane
    
    m=[223.15,2.37E-02,9.77E-06,0.8116;...
        273.15,3.04E-02,1.44E-05,0.7494;...
        323.15,3.77E-02,1.97E-05,0.7282;...
        373.15,4.53E-02,2.57E-05,0.7247;...
        423.15,5.34E-02,3.23E-05,0.7284;...
        473.15,6.19E-02,3.94E-05,0.7344;...
        573.15,8.00E-02,5.53E-05,0.745;...
        673.15,9.92E-02,7.30E-05,0.7501;...
        773.15,1.19E-01,9.23E-05,0.7502;...
        1273.15,2.26E-01,2.14E-04,0.7331;...
        1773.15,3.19E-01,4.02E-04,0.7936;...
        2273.15,3.68E-01,7.40E-04,1.0386];
    
elseif (fluid==4) % Water Vapor
    
    m=[223.15,1.35E-02,7.31E-06,1.0047;...
        273.15,1.67E-02,1.11E-05,1.0033;...
        323.15,2.03E-02,1.59E-05,0.9944;...
        373.15,2.43E-02,2.15E-05,0.983;...
        423.15,2.86E-02,2.81E-05,0.9712;...
        473.15,3.33E-02,3.56E-05,0.9599;...
        573.15,4.35E-02,5.34E-05,0.9401;...
        673.15,5.47E-02,7.50E-05,0.924;...
        773.15,6.68E-02,1.00E-04,0.9108;...
        1273.15,1.36E-01,2.76E-04,0.8639;...
        1773.15,2.13E-01,5.18E-04,0.8233;...
        2273.15,2.92E-01,8.08E-04,0.7833];
    
end

% Find Index
idx=1;
while idx<size(m,1) && temp>m(idx,1)
    idx=idx+1;
end
if (idx>1)
    idx=idx-1;
end

% Interpolate 
vals(1:3)=0.0;
if (idx>=size(m,1))
    idx=size(m,1);
    vals=m(idx,2:4);
elseif (idx==1)
    vals=m(idx,2:4);    
else
    % Loop and interpolate
    for i=1:3
        sl=m(idx+1,i+1)-m(idx,i+1);
        sl=sl/(m(idx+1,1)-m(idx,1));
        vals(i)=sl*(temp-m(idx,1))+m(idx,i+1);
    end
end

end
