function varargout = plot_basic(varargin)

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @plot_basic_OpeningFcn, ...
                   'gui_OutputFcn',  @plot_basic_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

% *************************************************************************
%  GUI Initialization - runs once!
% *************************************************************************
function plot_basic_OpeningFcn(hObject, eventdata, h, varargin)
     
    % Clear matlab terminal
    clc
        
    % Get arugments from live script, implied order:
    % diameter,k,length,temp left, temp right,resistance1,resistance2
    h.invars=cell2mat(varargin);
    
    % Check for number of arugments
    if(length(h.invars)~=12)
       disp('Need exactly 12 arguments')
       disp('Using hard-coded vars!')
       h.invars(1:12)=[1.0, ...         % wall thickness (mm)
                       4.0, ...         % cold flow ID (cm)
                       2.0, ...         % hot flow ID (cm)
                       1, ...           % flow type flag....0 or 1?
                       3e3, ...         % max length (cm?? UNITS)
                       1, ...           % k_wall INDEX 
                       1, ...           % cp hot flow INDEX
                       400.0, ...       % hot fluid entry temp (K)
                       1.0, ...        % hot fluid speed (cm/s)
                       2, ...           % cp hot flow INDEX
                       200.0, ...       % cold fluid entry temp (K)
                       1.0];           % cold fluid speed (cm/s)
    end
   
    plot_data_init(hObject,h);
    
end

end

% *************************************************************************
%  Data Initalization - Called at start and reset
% *************************************************************************
function plot_data_init(hObject,h)
    
    % Defined Global Constants
    h.nxi=50;
    nxi=h.nxi;
    
    % Set multipliers, values greater than 1 only
    h.mult_limit(1:12)=[2,2,2,2,2,2,2,2,2,2,2,2]; % set diameter limits
    
    % Initialize conductivity
    h.kwall_array=[0.02, 0.05, 0.1, 1.5, 2.22, 50.2,...
                   237, 310, 401, 1000];
    
    % Fluid Selection Properties           
    h.ccap_array=[4185.5,2400.0]; % J/(Kg*K)
    h.mu_array=[8.9e-4,6.0e-3]; % PI
    h.kfluid_array=[0.6,0.1e-3]; %W/(mK)
    h.rho_array=[1000.0,700.0]; % kg/m^3 
    
    % initialize node parameters (1:4)
    h.twall=h.invars(1)/1000.0; % wall thickness - converted to meters
    h.di_big=h.invars(2)/100.0; % cold/outer wall DI converted to meters
    h.di_sml=h.invars(3)/100.0; % hot/inner wall DI converted to meters
    h.flow_type=h.invars(4); % Flow Type flag
    h.max_length=h.invars(5); % max length covnerted to meters
    h.wallk=h.kwall_array(h.invars(6));
    h.ccap_hot =h.ccap_array(h.invars(7));
    h.ccap_cold=h.ccap_array(h.invars(10));
    h.mu_hot =h.mu_array(h.invars(7));
    h.mu_cold=h.mu_array(h.invars(10));
    h.kfluid_hot=h.kfluid_array(h.invars(7));
    h.kfluid_cold=h.kfluid_array(h.invars(10));
    h.rho_hot =h.rho_array(h.invars(7));
    h.rho_cold=h.rho_array(h.invars(10));
%     h.temp_hot=h.invars(8)+273.15; % Hot temp, converted to Kelvin
    h.temp_hot=h.invars(8); % Hot temp, converted to Kelvin
    h.vel_hot=h.invars(9)/100.0; % velcotiy, convered to m/s
%     h.temp_cold=h.invars(11)+273.15;
    h.temp_cold=h.invars(11);
    h.vel_cold=h.invars(12)/100.0;
    

    % Initialize Text output vars
    h.re_hot=0.0;
    h.re_cold=0.0;
    h.pr_hot=0.0;
    h.pr_cold=0.0;
    h.nu_hot=0.0;
    h.nu_cold=0.0;
    h.h_hot=0.0;
    h.h_cold=0.0;
    
    % Initialize mesh (length) array
    h.ex=zeros(nxi,1);
    h.ex=linspace(1.0e-20,h.max_length,h.nxi);    
    
    % Initialize output vars (1D)
    h.exit_hot_temp=zeros(nxi,1);
    h.exit_cold_temp=zeros(nxi,1);
    h.qflow=zeros(nxi,1);
    h.epsilon=zeros(nxi,1);
    
    % Define handle array (row on gui:column on gui)
    % note sparse in element rows!
    h.sliders=[h.thick_slider,h.di_cold_slider,h.di_hot_slider,...
               h.leng_slider,h.wallk_slider,0,h.thot_slider,h.uhot_slider,...
               0,h.tcold_slider,h.ucold_slider];
           
    h.lives=[h.thick_live,h.di_cold_live,h.di_hot_live,...
               h.leng_live,h.wallk_live,0,h.thot_live,h.uhot_live...
               0,h.tcold_live,h.ucold_live];
    
    h.klists=[0,0,0,0,h.wallk_list,0,0,0,0,0];
    
	set(h.wallk_list,'Value',6);
    set(h.fluid_hot_list,'Value',h.invars(7));
    set(h.fluid_cold_list,'Value',h.invars(10));
    
    % get initialization matrix (local scope)
    init(1:11)=[h.twall*1000.0,h.di_big*100.0,h.di_sml*100.0,h.max_length,...
               h.invars(6),0,h.temp_hot,h.vel_hot*100.0,0,h.temp_cold,h.vel_cold.*100.0];
    for j=[1:5,7:8,10:11]
        set(h.sliders(j),'Max',init(j)*h.mult_limit(j));
        set(h.sliders(j),'Min',init(j)/h.mult_limit(j));
        set(h.sliders(j),'Value',init(j));
        set(h.lives(j),'String',num2str(init(j)));
    end
    
    % Set correct Initial Push botton
    if (h.flow_type==1)
        set(h.button_pf_1,'Value',1);
        set(h.button_cf_2,'Value',0);
    else
        set(h.button_pf_1,'Value',0);
        set(h.button_cf_2,'Value',1);
    end
    
    % Get cross section
    h=update_cross_section(h);
    
    % Get data
    h=update_data(h);
    
    % Initialize Plots
    axes(h.temp_fig);  
    cla(h.temp_fig);
    cla reset
    grid on
    title('Hot and Cold Exit Temps as Function of Pipe Length')
    hold on
    yyaxis left
    h.plot1=plot( h.ex(:),h.exit_hot_temp(:)','r-' );
    ylabel([{'Hot Side'};{'Exit Temp (K)'}],'Color','r');
    ax=gca;
    set(ax(1),'YColor','r');
    yyaxis right
    h.plot2=plot( h.ex(:),h.exit_cold_temp(:)','b-' );
    ylabel([{'Cold Side'};{'Exit Temp (K)'}],'Color','b');     
    ax=gca;
    set(ax(1),'YColor','b');
    hold off
    
    % Plot qe Axes
    axes(h.qe_fig);
    cla(h.qe_fig);
    cla reset
    hold on
    xlabel('Heat Exchanger Length (m)')
%     yyaxis left
    h.plot3=plot( h.ex(:),h.qflow(:)','k');
    ylabel([{'Total Heat'};{'Exchange (W)'}],'Color','k');    
    grid on
    hold off
%     yyaxis right
%     h.plot4=plot( h.ex(:).*100.0,h.exit_cold_temp(:)' );
%     ylabel('Efficiency Coeff.');      
    % Generate new plots
    h=update_plot(h);
        
    % Choose default command line output for plot_basic
    h.output = hObject;
    
    % Update h structure
    guidata(hObject, h);
    
end


% *************************************************************************
%  Contact Resistance 
% *************************************************************************
function h=update_cross_section(h)
   
   % plot
   axes(h.cross_fig); 
   clear axes
   cla reset
   hold on
   r=0.5*h.di_big;
   r=r.*100.0;
   rectangle('Position',[-r,-r,2.0*r,2.0*r],...
             'FaceColor','b','EdgeColor','b',...
             'Curvature',[1 1]); % largest
   r=0.5*(h.di_sml+2.0*h.twall);           
   r=r.*100.0;
   rectangle('Position',[-r,-r,2.0*r,2.0*r],...
             'FaceColor','w','EdgeColor','w',...
             'Curvature',[1 1]); % outter of inner
   r=0.5*h.di_sml;
   r=r.*100.0;
   rectangle('Position',[-r,-r,2.0*r,2.0*r],...
             'FaceColor','r','EdgeColor','r',...
             'Curvature',[1 1]); % largest
   grid on
   axis equal
   hold off
%     
%    % Update h structure
%    guidata(hObject, h);
    
end

% *************************************************************************
%  DATA UPDATE - Generate new data piecmeal
% *************************************************************************
function h=update_data(h)
    
    % Get Prandlt numbers
    h.pr_hot =h.ccap_hot*h.mu_hot/h.kfluid_hot;
    h.pr_cold=h.ccap_cold*h.mu_cold/h.kfluid_cold;
    set(h.pr_hot_disp,'String',num2str2(h.pr_hot));
    set(h.pr_cold_disp,'String',num2str2(h.pr_cold));
    
    % Get Reynolds Numbers
    h.re_hot=h.rho_hot*h.vel_hot*(h.di_sml)/h.mu_hot;
    h.re_cold=h.rho_cold*h.vel_cold*...
               (h.di_big-(h.di_sml+2.0*h.twall))/h.mu_cold;
    set(h.re_hot_disp,'String',num2str2(h.re_hot));
    set(h.re_cold_disp,'String',num2str2(h.re_cold));    
    
    % Get Nusselt Numbers - From Dittus-Boelter Correlation
    h.nu_hot=0.023*(h.re_hot^(4.0/5.0))*(h.pr_hot^0.3);
    h.nu_cold=0.023*(h.re_cold^(4.0/5.0))*(h.pr_cold^0.4);
    set(h.nu_hot_disp,'String',num2str2(h.nu_hot));
    set(h.nu_cold_disp,'String',num2str2(h.nu_cold));      
    
    % Get h values
    h.h_hot=h.nu_hot*h.kfluid_hot/h.di_sml;
    h.h_cold=h.nu_cold*h.kfluid_cold/(h.di_big-(h.di_sml+2.0*h.twall));
    
    % Get epsilon(L/ex) to wall resistance
    h.epsilon(1:h.nxi)=log((h.di_sml+2.0*h.twall)/(h.di_sml));
    h.epsilon(1:h.nxi)=h.epsilon(1:h.nxi)./(2.0.*pi.*h.wallk.*h.ex(1:h.nxi)');
    disp(h.epsilon(10))
    % Get UA quanitity
    h.epsilon(1:h.nxi)=h.epsilon(1:h.nxi)+(1.0./(h.h_hot.*pi.*(h.di_sml).*h.ex'));
    h.epsilon(1:h.nxi)=h.epsilon(1:h.nxi)+(1.0./(h.h_cold.*pi.*(h.di_sml+2.0.*h.twall).*h.ex'));
    disp(h.epsilon(10))
    h.epsilon(1:h.nxi)=1.0./(h.epsilon(1:h.nxi));
    
    % Get heat capacity rate
    area_hot=pi*((h.di_sml/2.0)^2.0);
    mfr_hot=h.rho_hot.*h.vel_hot.*area_hot;
    h.C_hot=h.ccap_hot.*mfr_hot;
    area_cold=pi*((h.di_big/2.0)^2.0);
    area_cold=area_cold-pi*(((h.di_sml/2.0)+h.twall)^2.0);
    mfr_cold=h.rho_cold.*h.vel_cold.*area_cold;
    h.C_cold=h.ccap_cold.*mfr_cold;
    
    % Get NTU 
    h.epsilon=h.epsilon./min([h.C_hot,h.C_cold]);
    h.cr=min([h.C_hot,h.C_cold])/max([h.C_hot,h.C_cold]);
    
    % Get Epsilon precuroser
    if (h.flow_type==1) % Parallel Flow 
        h.epsilon(1:h.nxi)=(1.0-exp(-h.epsilon.*(1.0+h.cr)))/(1.0+h.cr);
    else % Counter Flow
        h.epsilon(1:h.nxi)=(1.0-exp(-h.epsilon(1:h.nxi).*(1.0-h.cr)))./...
            (1.0-h.cr.*exp(-h.epsilon(1:h.nxi).*(1.0-h.cr)));       
    end
    
    % get heating rate (Watts)
    h.qflow=h.epsilon.*min([h.C_hot,h.C_cold]).*...
            (h.temp_hot-h.temp_cold);
    
    % Get Temp Profiles
    h.exit_hot_temp=h.temp_hot-h.qflow./(mfr_hot.*h.ccap_hot);
    h.exit_cold_temp=h.temp_cold+h.qflow./(mfr_cold.*h.ccap_cold);
    
end

% *************************************************************************
%  PLOT UPDATE - Generate new plot
% *************************************************************************
function h=update_plot(h)

    % Plot Temp Axes
    axes(h.temp_fig);  
    hold on
    set(h.plot1,'XData',h.ex(:).*100.0);
    set(h.plot1,'YData',h.exit_hot_temp(:)');
    set(h.plot2,'XData',h.ex(:).*100.0);
    set(h.plot2,'YData',h.exit_cold_temp(:)');    
    hold off
    
    % Plot qe Axes
    axes(h.qe_fig);
    hold on
    set(h.plot3,'XData',h.ex(:).*100.0);
    set(h.plot3,'YData',h.qflow(:)');
%     set(h.plot4,'XData',h.ex(:).*100.0);
%     set(h.plot4,'YData',h.exit_cold_temp(:)');       
    hold off
    
end

% *************************************************************************
%  Update Thickness
% *************************************************************************
function h=update_twall(hObject, eventdata,h,dtype,idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.twall=get(h.sliders(idx),'Value');
    else % live text
        h.twall=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update value mm to meters
    h.twall=h.twall./1000.0;    
    
    % Test to ensure physical answer
    if (h.twall<=0.0 || h.di_sml>=h.di_big-2.0*h.twall)
        if (h.twall<=0.0)
            h.twall=1.0e-8;
        else
            h.twall=0.5.*(h.di_big-h.di_sml-1e-8);
        end
        if (dtype==0) % slider
            h.twall=h.twall.*1000.0;    
            h=update_entry(hObject, eventdata, h, idx, 1);
            h.twall=h.twall./1000.0; 
        else % live text
            h.twall=h.twall.*1000.0;    
            h=update_entry(hObject, eventdata, h, idx, 0);
            h.twall=h.twall./1000.0; 
        end
    end
    
    % Update edit box or slider
    h.twall=h.twall.*1000.0;    
    h=update_entry(hObject, eventdata, h, idx, dtype);
    h.twall=h.twall./1000.0;        
    
    % Update Cross section figure
    h=update_cross_section(h);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
    % Update h structure
    guidata(hObject, h);
    
end


% *************************************************************************
%  Outer Pipe Update
% *************************************************************************
function h=update_dcold(hObject, eventdata,h,dtype,idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.di_big=get(h.sliders(idx),'Value');
    else % live text
        h.di_big=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update value to meters
    h.di_big=h.di_big./100.0;

    % Test to ensure physical answer
    if (h.di_big<=0.0 || h.di_sml>=h.di_big-2.0*h.twall)
        h.di_big=h.di_sml+2.0*h.twall+1e-8;
        if (dtype==0) % slider
            h.di_big=h.di_big.*100.0;
            h=update_entry(hObject, eventdata, h, idx, 1);
            h.di_big=h.di_big./100.0;
        else % live text
            h.di_big=h.di_big.*100.0;
            h=update_entry(hObject, eventdata, h, idx, 0);
            h.di_big=h.di_big./100.0;
        end
    end
    
    % Update edit box or slider
    h.di_big=h.di_big.*100.0;
    h=update_entry(hObject, eventdata, h, idx, dtype);
    h.di_big=h.di_big./100.0;
    
    % Update Cross section figure
    h=update_cross_section(h);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
    % Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Inner Pipe Update
% *************************************************************************
function h=update_dhot(hObject, eventdata,h,dtype,idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.di_sml=get(h.sliders(idx),'Value');
    else % live text
        h.di_sml=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update value to meters
    h.di_sml=h.di_sml./100.0;
    
    % Test to ensure physical answer
    if (h.di_sml<=0.0 || h.di_sml>=h.di_big-2.0*h.twall)
        if (h.di_sml<=0.0)
            h.di_sml=1.0e-8;
        else
            h.di_sml=h.di_big-2.0*h.twall-1e-8;
        end
        if (dtype==0) % slider
            h.di_sml=h.di_sml.*100.0;
            h=update_entry(hObject, eventdata, h, idx, 1);
            h.di_sml=h.di_sml./100.0;
        else % live text
            h.di_sml=h.di_sml.*100.0;
            h=update_entry(hObject, eventdata, h, idx, 0);
            h.di_sml=h.di_sml./100.0;
        end
    end
    
    % Update edit box or slider
    h.di_sml=h.di_sml.*100.0;
    h=update_entry(hObject, eventdata, h, idx, dtype);
    h.di_sml=h.di_sml./100.0;

    % Update Cross section figure
    h=update_cross_section(h);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
    % Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Flow type
% *************************************************************************
function h=update_flow(hObject, eventdata,h,dtype)
    
    % update toggle switch value
    h.flow_type=dtype; 
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
    % Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Max Length
% *************************************************************************
function h=update_leng(hObject, eventdata,h,dtype,idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.max_length=get(h.sliders(idx),'Value');
    else % live text
        h.max_length=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % Update value
%     h.max_length=h.max_length./100.0;
    
    % update ex
    h.ex=linspace(1.0e-20,h.max_length,h.nxi);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
    % Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Wall Conductivity Value
% *************************************************************************
function h=update_wallk(hObject, eventdata, h, dtype, idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.wallk=get(h.sliders(idx),'Value');
    elseif (dtype==2) %list box
        h.wallk=klookup(get(h.klists(idx),'Value'),h);
    else % live text
        h.wallk=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Hot Fluid Value
% *************************************************************************
function h=update_fluid_hot_type(hObject, eventdata, h, dtype, idx)
    
    % get index of fluid type
    fluid_index=get(h.fluid_hot_list,'Value');
    h.ccap_hot   =h.ccap_array(fluid_index);
    h.mu_hot     =h.mu_array(fluid_index);
    h.kfluid_hot =h.kfluid_array(fluid_index);
    h.rho_hot    =h.rho_array(fluid_index);    
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Hot Fluid Temp
% *************************************************************************
function h=update_thot(hObject, eventdata, h, dtype, idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.temp_hot=get(h.sliders(idx),'Value');
    else % live text
        h.temp_hot=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % Update units c to k
%     h.temp_hot=h.temp_hot+273.15;
    h.temp_hot=h.temp_hot;
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Hot Fluid velocity
% *************************************************************************
function h=update_uhot(hObject, eventdata, h, dtype, idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.vel_hot=get(h.sliders(idx),'Value');
    else % live text
        h.vel_hot=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % Update units m/s
    h.vel_hot=h.vel_hot/100.0;
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Hot Fluid Value
% *************************************************************************
function h=update_fluid_cold_type(hObject, eventdata, h, dtype, idx)
    
    % get index of fluid type
    fluid_index=get(h.fluid_cold_list,'Value');
    h.ccap_cold   =h.ccap_array(fluid_index);
    h.mu_cold     =h.mu_array(fluid_index);
    h.kfluid_cold =h.kfluid_array(fluid_index);
    h.rho_cold    =h.rho_array(fluid_index);    
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Cold Fluid Temp
% *************************************************************************
function h=update_tcold(hObject, eventdata, h, dtype, idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.temp_cold=get(h.sliders(idx),'Value');
    else % live text
        h.temp_cold=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % Update units c to k
%     h.temp_cold=h.temp_cold+273.15;
    h.temp_cold=h.temp_cold;
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
%  Update Cold Fluid velocity
% *************************************************************************
function h=update_ucold(hObject, eventdata, h, dtype, idx)
    
    % update diamater & area
    if (dtype==0) % slider
        h.vel_cold=get(h.sliders(idx),'Value');
    else % live text
        h.vel_cold=str2num(get(h.lives(idx),'String')); 
    end
    
    % Update edit box or slider
    h=update_entry(hObject, eventdata, h, idx, dtype);
    
    % Update units m/s
    h.vel_cold=h.vel_cold/100.0;
    
    % call new data on all segment
    h=update_data(h);
    
    % replot all 
    h=update_plot(h);
        
	% Update h structure
    guidata(hObject, h);
    
end

% *************************************************************************
% *************************************************************************
%  Utility Functions
% *************************************************************************
% *************************************************************************

% *************************************************************************
% Generate conductiivty data
% *************************************************************************
function value=klookup(idx,h)

    % if custom, display original value
    if (idx>=length(h.kwall_array))
        value=h.kwall_array(end);
    else % get value from array
        value=h.kwall_array(idx);
    end
    
end


% *************************************************************************
%  Reset Data
% *************************************************************************
function reset_data_Callback(hObject, eventdata, h)
%     delete(h.plot1);
%     delete(h.plot2)
%     delete(h.plot3)
	plot_data_init(hObject,h);
end

% *************************************************************************
%  Make sure live text - slider - and popups all work in sync
% *************************************************************************
function h=update_entry(hObject, eventdata, h, idx, dtype)
    
    % if slider was changed
    if (dtype==0) % update live text, maybe popup
       
        % get value
        val=get(h.sliders(idx),'Value');
        
        % update live text
        set(h.lives(idx),'String',num2str2(val));
        
        % if thermal conducivity, set to custom
        if (idx==5) 
            set(h.klists(idx),'Value',11);
        end
        
    elseif (dtype==1) % if live was changed, update slider, maybe popup
        
        % get value
        val=str2num(get(h.lives(idx),'String'));
        
        % reset slider range if out of bounds
        h=slider_update(val,idx,h);
        
        % if thermal conducitivty, set to custom
        if (idx==5)
            set(h.klists(idx),'Value',11);
        end
        
    else % popup changed
        
        % get value
        if (idx==5) 
            val=klookup(get(h.klists(idx),'Value'),h);
        end
        
        % Reset slider range if out of bounds
        h=slider_update(val,idx,h);
        
        % fill live test field
        set(h.lives(idx),'String',num2str2(val));
        
    end
        
end

% *************************************************************************
%  Used in update_entry, common routine for updating sliders
% *************************************************************************
function h=slider_update(val,idx,h)
       
    % Get current rangers
    max_slider=get(h.sliders(idx),'Max');
    min_slider=get(h.sliders(idx),'Min');
    
    % If new value is out of bounds
    if (val>max_slider) % new value too big
        set(h.sliders(idx),'Max',val*h.mult_limit(idx)); 
        set(h.sliders(idx),'Value',val);
        set(h.sliders(idx),'Min',val/h.mult_limit(idx));
        
    elseif (val<min_slider) % New value too small
        set(h.sliders(idx),'Min',val/h.mult_limit(idx));
        set(h.sliders(idx),'Value',val);
        set(h.sliders(idx),'Max',val*h.mult_limit(idx));
        
    else % New value jsut right
        set(h.sliders(idx),'Value',val);
    end 

end

% *************************************************************************
%  Live Text - proper precision
% *************************************************************************
function str=num2str2(val)

    % Determine precision type
    if (val>10.0) % no decimals
        str=num2str(val,'%.0f');
    elseif (val>1.0) % 1 deciaml
        str=num2str(val,'%.1f');
    else % 8 preciision
        str=num2str(val,'%.3g');
    end
        
end

function varargout = plot_basic_OutputFcn(hObject, eventdata, h) 
varargout{1} = 0;
end


