function varargout = radiation(varargin)

    % Clear matlab terminal
    clc

    % Begin initialization code - DO NOT EDIT
    gui_Singleton = 1;
    gui_State = struct('gui_Name',       mfilename, ...
                       'gui_Singleton',  gui_Singleton, ...
                       'gui_OpeningFcn', @radiation_OpeningFcn, ...
                       'gui_OutputFcn',  @radiation_OutputFcn, ...
                       'gui_LayoutFcn',  [] , ...
                       'gui_Callback',   []);
    if nargin && ischar(varargin{1})
        gui_State.gui_Callback = str2func(varargin{1});
    end

    if nargout
        [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
    else
        gui_mainfcn(gui_State, varargin{:});
    end
    % End initialization code - DO NOT EDIT


% --- Executes just before fig is made visible.
function radiation_OpeningFcn(hObject, eventdata, h, varargin)
    

    %Get arugments from live script, implied order:
    h.invars=cell2mat(varargin);

    % Check for number of arugments
    if(length(h.invars)~=8)
       h.invars(1:8)=[4.0e-6,134.0,1.0,700.0,0.1,0.02,1.1458,1.0];
    end
    
    % Plot Image
    axes(h.axes2)
    matlabImage = imread('diagram.png');
    image(matlabImage)
    axis off
    axis image
   
    % Intialize plotting
    plot_data_init(hObject,h);
    
end
end


% Intitalize GUI & Data
function plot_data_init(hObject,h)

    % Inititalize Plot dimensiosn
    h.nxi=50;
    
    % Asssign intials states to all 
    h.input(1:8)=h.invars(1:8);
    
    % Create global data storage
    h.output=zeros(3,4); % row=row in gui
    h.ex(1:h.nxi)=zeros(h.nxi,1);
    h.ey(1:h.nxi)=zeros(h.nxi,1); 
    
    % Create callback topology
    h.fields_input=[h.detector_area,h.temp_shield,h.emissivity_shield, ...
             h.temp_sample,h.emissivity_sample,h.diameter_sample, ...
             h.theta_deg,h.length_t];
    
    % Create output topolgy
    h.fields_output=[h.dt_m,h.radio_temp,h.ds_m,h.temp_error];     
    
    % Set live values on all inputs
    for i=1:8
       set(h.fields_input(i),'String',num2str(h.input(i))); 
    end
    
    % Execute "Calculate" button
    h=calculate(h);
    
    % Choose default command line output for plot_basic
    h.output2 = hObject;
    
    % Update h structure
    guidata(hObject, h);
    
end
        
% intermediate for button press
function h=calculate_button(hObject,eventdata,h)
    h=calculate(h);
end

% Functin Callback for Running Code
function h=calculate(h)
    
    % Constants
    sigma = 5.6704e-8; %W/m^2/K^4, Stefan-Boltzman constant

    % Grab Sample properties
    Ad        = h.input(1); %m^2, detector area
    Tsh       = h.input(2); %K, shield temperature
    emiss_sh  = h.input(3); %emissivity of shield
    Ts        = h.input(4); %K, sample temperature
    emiss_s   = h.input(5); %emissivity of sample
    Ds        = h.input(6); %meter, diameter of sample
    theta_d   = h.input(7); %0.1;   %degrees, the divergence angle of the detector
    Lt        = h.input(8); %meter, distance from detector to sample
    
    %d Dameter of detector-observed diameter @ sample plane
    Dt = 2*Lt*tand(theta_d); 
    
    % Calculate detector-observed areas of the shield and the sample
    if Dt < Ds %detector sees only sample and no sheild
        As = pi/4*Dt^2; %m^2, sample area observed
        Ash = 0; % shield area observed
    else
        As = pi/4*Ds^2; %m^2, sample area observed
        Ash = pi/4*Dt^2 - As; %m^2, shield area observed
    end;
    
    % Areas
    % As = area of sample oberved by detector, m^2
    Atotal = As + Ash;
    As_pct = As/Atotal*100; % percentage of detector observed area comprised of sample
    Ash_pct = Ash/Atotal*100; %percentage of detector observed area comprised of shield
    
    % calculate heat emitted from both sample and shield that reach detector
    qs_d = emiss_s*sigma*Ts^4*As*Ad/pi/Lt^2; %Watts, heat transfer from sample detected by detector
    
    % Detector observaion area @ sample plane smaller than shield diameter
    if Dt < Ds 
        qsh_d = 0; %Watts, detector does not see shield
    else
        qsh_d = sigma*Tsh^4/pi*Ash*(Lt/(((Ds+Dt)/4)^2+Lt^2)^(1/2))^2 *...
                Ad/(((Ds+Dt)/4)^2+Lt^2); %, Watts, ht from shield detected
    end
    
    % Catch error in areas
    if Dt < Ds
        disp('NOTE, DETECTOR SEES ONLY SAMPLE AND NO PORTION OF SHIELD');
    end
    
    % Final Variables
    qtotal = qs_d + qsh_d; %sum of radiation from sample & shield seen by detector
    qs_pct = qs_d/qtotal*100; %percentage radiation reaching detector from sample
    qsh_pct = qsh_d/qtotal*100;%percentage radiation reaching detector from shield

    Ts_measured = ( (pi*qtotal*Lt^2) / (emiss_s*sigma*As*Ad))^(1/4); %radiometer-measured temperature
    Ts_error_pct = abs((Ts_measured-Ts)/Ts)*100; %percentage error in measured temperature

    % Text Outputs [h.dt_m,h.radio_temp,h.ds_m,h.temp_error];    
    array1=[Dt,Ts_measured,Ds,Ts_error_pct];
    for i=1:4
        set(h.fields_output(i),'String',num2str(array1(i)));
    end
    
    % Table output
    h.output(1,1:4)=[As, As_pct, qs_d, qs_pct]; % First row
    h.output(2,1:4)=[Ash, Ash_pct, qsh_d, qsh_pct]; % 2nd row
    h.output(3,1:4)=[As+Ash, As_pct+Ash_pct, qs_d+qsh_d, qs_pct+qsh_pct]; % 3rd row
    set(h.table1,'Data',h.output);
    
    % Plot
    axes(h.axes1);
    cla reset
    hold on;

    % plot detector viewing divergence
    plot([0 0], [0, Lt], 'k--'); %center line
    plot([-1*Dt/2 0], [0 Lt], 'k--'); %left divergence line
    plot([1*Dt/2 0], [0 Lt], 'k--'); %right divergence line

    % draw sample
    a = xlim;
    b = ylim;
    Sample_thick = (b(2)-b(1))/30;
    rectangle('Position',[0-Ds/2 -2*Sample_thick Ds Sample_thick],...
               'FaceColor',[1 0 0]);
    a = xlim; %update width scale after plotting sample
    text(0,-3*Sample_thick,'Sample','HorizontalAlignment','center')

    % draw detector
    Detect_width = (a(2)-a(1))/5;
    Detect_ht = (b(2)-b(1))/5;
    rectangle('Position',[0-Detect_width/2 Lt Detect_width Detect_ht],...
              'FaceColor',[0 1 0]);
    text(0,Lt+Detect_ht/2,'Detector','HorizontalAlignment','center')

    % draw shield
    if Dt > Ds %if detector views both sample and shield
        rectangle('Position',[a(1) -1*Sample_thick (abs(a(1))-Ds/2)...
                  Sample_thick], 'FaceColor',[0 0 1]);
        rectangle('Position',[Ds/2 -1*Sample_thick (abs(a(1))-Ds/2)...
                  Sample_thick], 'FaceColor',[0 0 1]);
        text((a(1)+-1*Ds/2)/2,-2*Sample_thick,'Shield',...
             'HorizontalAlignment','center')
        text((a(2)+Ds/2)/2,-2*Sample_thick,'Shield',...
             'HorizontalAlignment','center')
    end
    % if detector only views sample, do not plot shield.
    hold off;
    
    
end

% Function for Live-Text Entry
function h=update_fig(hObject,eventdata,h,idx)
    
    % Update input vector
    str=str2double(get(h.fields_input(idx),'String'));
    if (isnumeric(str)==1)
        h.input(idx)=str;
    end
    set(h.fields_input(idx),'String',num2str(h.input(idx)));
    
    % Update h structure
    guidata(hObject, h);
    
end


% uneeded output function -- DONT Erase, MATLAB is POS
function radiation_OutputFcn(hObject, eventdata, h, varargin)
end
